# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import logging
import torch
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torch import autocast
from transformers.utils import is_remote_url
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.peft import (
    ClipLoraForMatching as _ClipLoraForMatching,
    ClipLoraForTextMatching as _ClipLoraForTextMatching,
)
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_model,
)
from unitorch.cli.models import generation_model_decorator
from unitorch.cli.models import ClassificationOutputs, GenerationOutputs, LossOutputs
from unitorch.cli.models.clip import pretrained_clip_infos


@register_model("core/model/matching/peft/lora/clip")
class ClipLoraForMatching(_ClipLoraForMatching):
    def __init__(
        self,
        config_path: str,
        projection_dim: Optional[int] = 512,
        lora_r: Optional[int] = 16,
        lora_alpha: Optional[int] = 32,
        lora_dropout: Optional[float] = 0.05,
        fan_in_fan_out: Optional[bool] = True,
        target_modules: Optional[Union[List[str], str]] = ["q_proj", "v_proj"],
    ):
        """
        Initialize the BloomLoraForClassification model.

        Args:
            config_path (str): The path to the model configuration file.
            lora_r (int, optional): The number of Lora ranks. Defaults to 16.
            lora_alpha (int, optional): The Lora alpha value. Defaults to 32.
            lora_dropout (float, optional): The Lora dropout rate. Defaults to 0.05.
            fan_in_fan_out (bool, optional): Whether to use fan-in/fan-out weight initialization. Defaults to True.
            target_modules (Union[List[str], str], optional): The target modules for Lora regularization. Defaults to ["q_proj", "v_proj"].
            num_classes (int, optional): The number of classes. Defaults to 1.
            gradient_checkpointing (bool, optional): Whether to use gradient checkpointing during training. Defaults to False.
        """
        super().__init__(
            config_path=config_path,
            projection_dim=projection_dim,
            lora_r=lora_r,
            lora_alpha=lora_alpha,
            lora_dropout=lora_dropout,
            fan_in_fan_out=fan_in_fan_out,
            target_modules=target_modules,
        )

    @classmethod
    @add_default_section_for_init("core/model/matching/peft/lora/clip")
    def from_core_configure(cls, config, **kwargs):
        """
        Create an instance of BloomLoraForClassification from a core configuration.

        Args:
            config: The core configuration.
            **kwargs: Additional keyword arguments.

        Returns:
            BloomLoraForClassification: The initialized BloomLoraForClassification instance.
        """
        config.set_default_section("core/model/matching/peft/lora/clip")
        pretrained_name = config.getoption("pretrained_name", "clip-vit-base-patch16")
        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "config"),
        )
        config_path = cached_path(config_path)
        projection_dim = config.getoption("projection_dim", 512)

        lora_r = config.getoption("lora_r", 16)
        lora_alpha = config.getoption("lora_alpha", 32)
        lora_dropout = config.getoption("lora_dropout", 0.05)
        fan_in_fan_out = config.getoption("fan_in_fan_out", True)
        target_modules = config.getoption("target_modules", ["q_proj", "v_proj"])

        inst = cls(
            config_path,
            projection_dim=projection_dim,
            lora_r=lora_r,
            lora_alpha=lora_alpha,
            lora_dropout=lora_dropout,
            fan_in_fan_out=fan_in_fan_out,
            target_modules=target_modules,
        )

        weight_path = []
        pretrained_weight_path = config.getoption("pretrained_weight_path", None)
        pretrained_weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_clip_infos, pretrained_name, "weight"),
            check_none=False,
        )
        if pretrained_weight_path is not None:
            if isinstance(pretrained_weight_path, str):
                weight_path.append(pretrained_weight_path)
            elif isinstance(pretrained_weight_path, list):
                weight_path.extend(pretrained_weight_path)

        pretrained_lora_weight_path = config.getoption(
            "pretrained_lora_weight_path", None
        )
        if pretrained_lora_weight_path is not None:
            weight_path.append(pretrained_lora_weight_path)

        if len(weight_path) > 0:
            inst.from_pretrained(
                weight_path=weight_path,
            )

        return inst

    @autocast(device_type=("cuda" if torch.cuda.is_available() else "cpu"))
    def forward(
        self,
        input_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        attention_mask: torch.Tensor,
        position_ids: torch.Tensor,
    ):
        """
        Perform forward pass of the BloomLoraForClassification model.

        Args:
            input_ids (torch.Tensor): The input IDs.
            attention_mask (torch.Tensor, optional): The attention mask.
            position_ids (torch.Tensor, optional): The position IDs.

        Returns:
            ClassificationOutputs: The output of the classification task.
        """
        outputs = super().forward(
            input_ids=input_ids,
            pixel_values=pixel_values,
            attention_mask=attention_mask,
            position_ids=position_ids,
        )
        return ClassificationOutputs(outputs=outputs)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli import hf_endpoint_url

# pretrained infos
pretrained_segformer_infos = {
    "segformer-b2-human-parse-24": {
        "config": hf_endpoint_url(
            "/yolo12138/segformer-b2-human-parse-24/resolve/main/config.json"
        ),
        "vision_config": hf_endpoint_url(
            "/yolo12138/segformer-b2-human-parse-24/resolve/main/preprocessor_config.json"
        ),
        "weight": hf_endpoint_url(
            "/yolo12138/segformer-b2-human-parse-24/resolve/main/model.safetensors"
        ),
    },
    "segformer-b5-finetuned-human-parsing": {
        "config": hf_endpoint_url(
            "/matei-dorian/segformer-b5-finetuned-human-parsing/resolve/main/config.json"
        ),
        "vision_config": hf_endpoint_url(
            "/matei-dorian/segformer-b5-finetuned-human-parsing/resolve/main/preprocessor_config.json"
        ),
        "weight": hf_endpoint_url(
            "/matei-dorian/segformer-b5-finetuned-human-parsing/resolve/main/model.safetensors"
        ),
    },
    "segformer-face-parsing": {
        "config": hf_endpoint_url(
            "/jonathandinu/face-parsing/resolve/main/config.json"
        ),
        "vision_config": hf_endpoint_url(
            "/jonathandinu/face-parsing/resolve/main/preprocessor_config.json"
        ),
        "weight": hf_endpoint_url(
            "/jonathandinu/face-parsing/resolve/main/model.safetensors"
        ),
    },
}

import unitorch.cli.models.segformer.modeling
import unitorch.cli.models.segformer.processing
from unitorch.cli.models.segformer.modeling import SegformerForSegmentation
from unitorch.cli.models.segformer.processing import SegformerProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import cv2
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.diffusers import WanProcessor as _WanProcessor
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli.models import (
    TensorsInputs,
)
from unitorch.cli.models.diffusers import pretrained_stable_infos


class WanProcessor(_WanProcessor):
    def __init__(
        self,
        vocab_path: str,
        vae_config_path: Optional[str] = None,
        image_config_path: Optional[str] = None,
        max_seq_length: Optional[int] = 77,
        position_start_id: Optional[int] = 0,
        video_size: Optional[Tuple[int, int]] = None,
    ):
        super().__init__(
            vocab_path=vocab_path,
            vae_config_path=vae_config_path,
            image_config_path=image_config_path,
            max_seq_length=max_seq_length,
            position_start_id=position_start_id,
            video_size=video_size,
        )

    @classmethod
    @add_default_section_for_init("core/process/diffusion/wan")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/process/diffusion/wan")
        pretrained_name = config.getoption("pretrained_name", "wan-v2.1-i2v-14b")
        pretrained_infos = nested_dict_value(pretrained_stable_infos, pretrained_name)

        vocab_path = config.getoption("vocab_path", None)
        vocab_path = pop_value(
            vocab_path,
            nested_dict_value(pretrained_infos, "text", "vocab"),
        )
        vocab_path = cached_path(vocab_path)

        vae_config_path = config.getoption("vae_config_path", None)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrained_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)

        image_config_path = config.getoption("image_config_path", None)
        image_config_path = pop_value(
            image_config_path,
            nested_dict_value(pretrained_infos, "image", "vision_config"),
            check_none=False,
        )
        if image_config_path is not None:
            image_config_path = cached_path(image_config_path)

        return {
            "vocab_path": vocab_path,
            "vae_config_path": vae_config_path,
            "image_config_path": image_config_path,
        }

    @register_process("core/process/diffusion/wan/text2video")
    def _text2video(
        self,
        prompt: str,
        video: Union[cv2.VideoCapture, str, List[Image.Image]],
        max_seq_length: Optional[int] = None,
    ):
        outputs = super().text2video(
            prompt=prompt,
            video=video,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(
            pixel_values=outputs.pixel_values,
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
        )

    @register_process("core/process/diffusion/wan/text2video/inputs")
    def _text2video_inputs(
        self,
        prompt: str,
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        outputs = super().text2video_inputs(
            prompt=prompt,
            negative_prompt=negative_prompt,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(
            input_ids=outputs.input_ids,
            negative_input_ids=outputs.negative_input_ids,
            attention_mask=outputs.attention_mask,
            negative_attention_mask=outputs.negative_attention_mask,
        )

    @register_process("core/process/diffusion/wan/image2video")
    def _image2video(
        self,
        prompt: str,
        video: Union[cv2.VideoCapture, str, List[Image.Image]],
        image: Optional[Union[Image.Image, str]] = None,
        max_seq_length: Optional[int] = None,
    ):
        outputs = super().image2video(
            prompt=prompt,
            video=video,
            image=image,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(
            pixel_values=outputs.pixel_values,
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            condition_pixel_values=outputs.condition_pixel_values,
            vae_pixel_values=outputs.vae_pixel_values,
        )

    @register_process("core/process/diffusion/wan/image2video/inputs")
    def _image2video_inputs(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        outputs = super().image2video_inputs(
            prompt=prompt,
            image=image,
            negative_prompt=negative_prompt,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(
            condition_pixel_values=outputs.condition_pixel_values,
            vae_pixel_values=outputs.vae_pixel_values,
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            negative_input_ids=outputs.negative_input_ids,
            negative_attention_mask=outputs.negative_attention_mask,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import torch.nn as nn
import pyarrow as pa
from dataclasses import dataclass
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.cli import (
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli import WriterMixin, WriterOutputs
from unitorch.cli.models import TensorsOutputs, TensorsTargets, ACT2FN


@dataclass
class ClassificationOutputs(TensorsOutputs, WriterMixin):
    """
    Outputs for classification models.

    Args:
        outputs (torch.Tensor): Output tensor containing the classification results.
    """

    outputs: torch.Tensor


@dataclass
class EmbeddingOutputs(TensorsOutputs, WriterMixin):
    """
    Outputs for embedding models.

    Args:
        embedding (torch.Tensor): The embedding tensor.
        embedding1 (Optional[torch.Tensor]): Additional embedding tensor 1. Defaults to an empty tensor.
        embedding2 (Optional[torch.Tensor]): Additional embedding tensor 2. Defaults to an empty tensor.
        embedding3 (Optional[torch.Tensor]): Additional embedding tensor 3. Defaults to an empty tensor.
        embedding4 (Optional[torch.Tensor]): Additional embedding tensor 4. Defaults to an empty tensor.
    """

    embedding: torch.Tensor
    embedding1: Optional[torch.Tensor] = torch.empty(0)
    embedding2: Optional[torch.Tensor] = torch.empty(0)
    embedding3: Optional[torch.Tensor] = torch.empty(0)
    embedding4: Optional[torch.Tensor] = torch.empty(0)


@dataclass
class ClassificationTargets(TensorsTargets):
    """
    Targets for classification models.

    Args:
        targets (torch.Tensor): The target tensor.
        sample_weight (Optional[torch.Tensor]): The weight associated with each target. Defaults to a tensor with a value of 1.0.
    """

    targets: torch.Tensor
    sample_weight: Optional[torch.Tensor] = torch.tensor(1.0)


class ClassificationProcessor:
    """Processor for classification models."""

    def __init__(
        self,
        act_fn: Optional[str] = None,
        return_scores: Optional[bool] = False,
        id2label: Optional[Dict[int, str]] = None,
    ):
        """
        Initialize the ClassificationProcessor.

        Args:
            act_fn (Optional[str]): Activation function to apply to the model outputs.
            return_scores (Optional[bool]): Whether to return the scores in addition to the predictions.
        """
        self.act_fn = ACT2FN.get(act_fn, None)
        self.return_scores = return_scores
        self.id2label = id2label

    @classmethod
    @add_default_section_for_init("core/process/classification")
    def from_core_configure(cls, config, **kwargs):
        """
        Create a ClassificationProcessor instance from core configuration.

        Args:
            config: Configuration object.
            **kwargs: Additional keyword arguments.

        Returns:
            ClassificationProcessor: The initialized ClassificationProcessor instance.
        """
        pass

    @register_process("core/postprocess/classification/binary_score")
    def _binary_score(
        self,
        outputs: ClassificationOutputs,
    ):
        """
        Postprocess the classification outputs for binary classification with scores.

        Args:
            outputs (ClassificationOutputs): Outputs from the classification model.

        Returns:
            WriterOutputs: Processed outputs with scores.
        """
        assert outputs.outputs.dim() == 2

        results = outputs.to_pandas()
        assert results.shape[0] == 0 or results.shape[0] == outputs.outputs.shape[0]

        outputs = outputs.outputs.numpy()
        if self.act_fn is not None:
            outputs = self.act_fn(outputs)

        if outputs.ndim == 2:
            pscore = outputs[:, 1] if outputs.shape[-1] > 1 else outputs[:, 0]
            results["pscore"] = pscore.tolist()
        else:
            results["pscore"] = outputs.tolist()
        return WriterOutputs(results)

    @register_process("core/postprocess/classification/score")
    def _classifier_score(
        self,
        outputs: ClassificationOutputs,
    ):
        """
        Postprocess the classification outputs for multi-classes classification with scores.

        Args:
            outputs (ClassificationOutputs): Outputs from the classification model.

        Returns:
            WriterOutputs: Processed outputs with scores and predicted classes.
        """
        assert outputs.outputs.dim() == 2

        results = outputs.to_pandas()
        assert results.shape[0] == 0 or results.shape[0] == outputs.outputs.shape[0]
        outputs = outputs.outputs.numpy()
        if self.act_fn is not None:
            outputs = self.act_fn(outputs)

        results["pscore"] = outputs.max(-1)
        results["pclass"] = outputs.argmax(-1)
        if self.id2label is not None:
            results["pclass"] = results["pclass"].map(self.id2label)

        if self.return_scores:
            results["scores"] = outputs.tolist()

        return WriterOutputs(results)

    @register_process("core/postprocess/classification/embedding")
    def _embedding(
        self,
        outputs: EmbeddingOutputs,
    ):
        """
        Postprocess the embedding outputs.

        Args:
            outputs (EmbeddingOutputs): Outputs from the embedding model.

        Returns:
            WriterOutputs: Processed outputs with embeddings.
        """
        results = outputs.to_pandas()
        assert results.shape[0] == 0 or results.shape[0] == outputs.embedding.shape[0]

        embedding = outputs.embedding.numpy()
        if embedding.ndim > 2:
            embedding = embedding.reshape(embedding.size(0), -1)
        results["embedding"] = embedding.tolist()

        embedding1 = outputs.embedding1.numpy()
        if embedding1.size > 0:
            if embedding1.ndim > 2:
                embedding1 = embedding1.reshape(embedding1.size(0), -1)
            results["embedding1"] = embedding1.tolist()

        embedding2 = outputs.embedding2.numpy()
        if embedding2.size > 0:
            if embedding2.ndim > 2:
                embedding2 = embedding2.reshape(embedding2.size(0), -1)
            results["embedding2"] = embedding2.tolist()

        embedding3 = outputs.embedding3.numpy()
        if embedding3.size > 0:
            if embedding3.ndim > 2:
                embedding3 = embedding3.reshape(embedding3.size(0), -1)
            results["embedding3"] = embedding3.tolist()

        embedding4 = outputs.embedding4.numpy()
        if embedding4.size > 0:
            if embedding4.ndim > 2:
                embedding4 = embedding4.reshape(embedding4.size(0), -1)
            results["embedding4"] = embedding4.tolist()

        return WriterOutputs(results)

    @register_process("core/postprocess/classification/embedding/string")
    def _embedding_string(
        self,
        outputs: EmbeddingOutputs,
    ):
        """
        Postprocess the embedding outputs as string representations.

        Args:
            outputs (EmbeddingOutputs): Outputs from the embedding model.

        Returns:
            WriterOutputs: Processed outputs with string representations of embeddings.
        """
        results = outputs.to_pandas()
        assert results.shape[0] == 0 or results.shape[0] == outputs.embedding.shape[0]

        embedding = outputs.embedding.numpy()
        if embedding.ndim > 2:
            embedding = embedding.reshape(embedding.size(0), -1)
        results["embedding"] = embedding.tolist()
        results["embedding"] = results["embedding"].map(
            lambda x: " ".join([str(i) for i in x])
        )

        embedding1 = outputs.embedding1.numpy()
        if embedding1.size > 0:
            if embedding1.ndim > 2:
                embedding1 = embedding1.reshape(embedding1.size(0), -1)
            results["embedding1"] = embedding1.tolist()
            results["embedding1"] = results["embedding1"].map(
                lambda x: " ".join([str(i) for i in x])
            )

        embedding2 = outputs.embedding2.numpy()
        if embedding2.size > 0:
            if embedding2.ndim > 2:
                embedding2 = embedding2.reshape(embedding2.size(0), -1)
            results["embedding2"] = embedding2.tolist()
            results["embedding2"] = results["embedding2"].map(
                lambda x: " ".join([str(i) for i in x])
            )

        embedding3 = outputs.embedding3.numpy()
        if embedding3.size > 0:
            if embedding3.ndim > 2:
                embedding3 = embedding3.reshape(embedding3.size(0), -1)
            results["embedding3"] = embedding3.tolist()
            results["embedding3"] = results["embedding3"].map(
                lambda x: " ".join([str(i) for i in x])
            )

        embedding4 = outputs.embedding4.numpy()
        if embedding4.size > 0:
            if embedding4.ndim > 2:
                embedding4 = embedding4.reshape(embedding4.size(0), -1)
            results["embedding4"] = embedding4.tolist()
            results["embedding4"] = results["embedding4"].map(
                lambda x: " ".join([str(i) for i in x])
            )

        return WriterOutputs(results)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import hashlib
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from PIL import Image
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.sam import SamProcessor as _SamProcessor
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli import WriterOutputs
from unitorch.cli.models import SegmentationOutputs, TensorsInputs
from unitorch.cli.models.sam import pretrained_sam_infos


class SamProcessor(_SamProcessor):
    def __init__(
        self,
        vision_config_path: str,
        output_folder: Optional[str] = None,
    ):
        super().__init__(
            vision_config_path=vision_config_path,
        )
        assert output_folder is not None
        self.output_folder = output_folder
        if not os.path.exists(output_folder):
            os.makedirs(self.output_folder, exist_ok=True)

    @classmethod
    @add_default_section_for_init("core/process/sam")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/process/sam")
        pretrained_name = config.getoption("pretrained_name", "sam-vit-base")
        vision_config_path = config.getoption("vision_config_path", None)
        vision_config_path = pop_value(
            vision_config_path,
            nested_dict_value(pretrained_sam_infos, pretrained_name, "vision_config"),
        )

        vision_config_path = cached_path(vision_config_path)

        return {
            "vision_config_path": vision_config_path,
        }

    def save(self, image: Image.Image):
        md5 = hashlib.md5()
        md5.update(image.tobytes())
        name = md5.hexdigest() + ".jpg"
        image.save(f"{self.output_folder}/{name}")
        return name

    @register_process("core/process/sam/segmentation/inputs")
    def _segmentation_inputs(
        self,
        image: Union[Image.Image, str],
        points_per_crop: Optional[int] = 32,
    ):
        if isinstance(image, str):
            image = Image.open(image)
        outputs = super().segmentation_inputs(
            image=image,
            points_per_crop=points_per_crop,
        )
        return TensorsInputs(
            pixel_values=outputs.pixel_values,
            original_sizes=outputs.original_sizes,
            reshaped_input_sizes=outputs.reshaped_input_sizes,
            input_points=outputs.input_points,
            input_labels=outputs.input_labels,
            input_boxes=outputs.input_boxes,
        )

    @register_process("core/postprocess/sam/segmentation")
    def _processing_masks(self, outputs: SegmentationOutputs):
        results = outputs.to_pandas()
        assert results.shape[0] == 0 or results.shape[0] == len(outputs.masks)
        results["mask_image"] = [
            ";".join([self.save(Image.fromarray(_m_.numpy())) for _m_ in m])
            for m in outputs.outputs
        ]
        return WriterOutputs(results)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import re
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union


def matched_pretrained_names(
    pretrained_names: List[str],
    match_patterns: Union[str, List[str]],
    block_patterns: Optional[Union[str, List[str]]] = None,
) -> List[str]:
    if isinstance(match_patterns, str):
        match_patterns = [match_patterns]
    if block_patterns is None:
        block_patterns = []
    elif isinstance(block_patterns, str):
        block_patterns = [block_patterns]
    matched = [
        p for p in pretrained_names if any([re.match(m, p) for m in match_patterns])
    ]
    blacked = [p for p in matched if not any([re.match(b, p) for b in block_patterns])]
    return blacked


from unitorch.utils import (
    is_diffusers_available,
    is_opencv_available,
)
from unitorch.cli import GenericWebUI
from unitorch.cli import CoreConfigureParser
from unitorch.cli.pipelines import Schedulers

supported_scheduler_names = list(Schedulers.keys())


class SimpleWebUI(GenericWebUI):
    def __init__(self, config: CoreConfigureParser, iname="Simple", iface=None):
        self._config = config
        self._iname = iname
        self._iface = iface

    @property
    def iname(self):
        return self._iname

    @property
    def iface(self):
        return self._iface

    def start(self):
        pass

    def stop(self):
        pass


from unitorch.cli.webuis.utils import (
    create_element,
    create_accordion,
    create_row,
    create_column,
    create_flex_layout,
    create_group,
    create_tab,
    create_tabs,
    create_blocks,
    create_pretrain_layout,
    create_controlnet_layout,
    create_adapter_layout,
    create_lora_layout,
)

import unitorch.cli.webuis.tools
import unitorch.cli.webuis.blip
import unitorch.cli.webuis.bloom
import unitorch.cli.webuis.bria
import unitorch.cli.webuis.clip
import unitorch.cli.webuis.detr
import unitorch.cli.webuis.dpt
import unitorch.cli.webuis.grounding_dino
import unitorch.cli.webuis.llama
import unitorch.cli.webuis.llava.mistral_clip
import unitorch.cli.webuis.mask2former
import unitorch.cli.webuis.mistral
import unitorch.cli.webuis.sam
import unitorch.cli.webuis.segformer

if is_diffusers_available():
    import unitorch.cli.webuis.stable
    import unitorch.cli.webuis.stable_xl

    import unitorch.cli.webuis.stable_3
    import unitorch.cli.webuis.stable_flux

    if is_opencv_available():
        import unitorch.cli.webuis.wan



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torch import autocast
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.grounding_dino import (
    GroundingDinoForDetection as _GroundingDinoForDetection,
)
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_model,
)
from unitorch.cli.models import (
    detection_model_decorator,
    segmentation_model_decorator,
    DetectionOutputs,
    SegmentationOutputs,
    LossOutputs,
)
from unitorch.cli.models.grounding_dino import pretrained_grounding_dino_infos


@register_model("core/model/detection/grounding_dino", detection_model_decorator)
class GroundingDinoForDetection(_GroundingDinoForDetection):
    def __init__(
        self,
        config_path: str,
    ):
        super().__init__(
            config_path=config_path,
        )

    @classmethod
    @add_default_section_for_init("core/model/detection/grounding_dino")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/model/detection/grounding_dino")
        pretrained_name = config.getoption("pretrained_name", "grounding-dino-tiny")
        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(
                pretrained_grounding_dino_infos, pretrained_name, "config"
            ),
        )

        config_path = cached_path(config_path)

        inst = cls(
            config_path=config_path,
        )
        pretrained_weight_path = config.getoption("pretrained_weight_path", None)
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(
                pretrained_grounding_dino_infos, pretrained_name, "weight"
            ),
            check_none=False,
        )
        if weight_path is not None:
            inst.from_pretrained(weight_path)

        return inst

    @autocast(device_type=("cuda" if torch.cuda.is_available() else "cpu"))
    def forward(
        self, pixel_values, input_ids, attention_mask, token_type_ids, bboxes, classes
    ):
        outputs = super().forward(
            pixel_values=pixel_values,
            input_ids=input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            bboxes=bboxes,
            classes=classes,
        )
        return LossOutputs(loss=outputs)

    @add_default_section_for_function("core/model/detection/grounding_dino")
    def detect(
        self,
        pixel_values,
        input_ids,
        attention_mask,
        token_type_ids,
        norm_bboxes: bool = False,
        text_threshold: Optional[float] = 0.25,
        box_threshold: Optional[float] = 0.25,
    ):
        outputs = super().detect(
            pixel_values=pixel_values,
            input_ids=input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            norm_bboxes=norm_bboxes,
            text_threshold=text_threshold,
            box_threshold=box_threshold,
        )
        classes = [c.max(dim=-1)[1] for c in outputs.classes]
        return DetectionOutputs(
            bboxes=outputs.bboxes,
            scores=outputs.scores,
            classes=classes,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import time
import logging
import http.server
import zipfile
from urllib.parse import parse_qs, urlparse
from threading import Thread
from functools import lru_cache
from unitorch.cli import CoreConfigureParser
from unitorch.cli import register_service, GenericService


def get_zipfile(zfs, res, idx, step):
    """
    Extracts a subset of zip files from the given list.

    Args:
        zfs (list): List of zip files.
        res (list): Result list to store extracted zip files.
        idx (int): Starting index.
        step (int): Step size.

    Returns:
        None
    """
    for i in range(idx, len(zfs), step):
        res[i] = zipfile.ZipFile(zfs[i])


def get_zipfiles(zipfiles, num_thread=48):
    """
    Extracts multiple zip files using multiple threads.

    Args:
        zipfiles (list): List of zip file paths.
        num_thread (int): Number of threads to use.

    Returns:
        list: Extracted zip files.
    """
    num_thread = min(len(zipfiles), num_thread)
    threads = [None] * num_thread
    results = [None] * len(zipfiles)
    for i in range(len(threads)):
        threads[i] = Thread(
            target=get_zipfile,
            args=(zipfiles, results, i, num_thread),
        )
        threads[i].start()

    for i in range(len(threads)):
        threads[i].join()
    return results


def parse_params(path):
    """
    Parses query parameters from the URL path.

    Args:
        path (str): URL path.

    Returns:
        dict: Parsed query parameters.
    """
    url_components = urlparse(path)
    query_params = parse_qs(url_components.query)
    return {k: v[0] for k, v in query_params.items()}


class ZipFilesServer(http.server.BaseHTTPRequestHandler):
    """
    HTTP request handler class.
    """

    zip_data = None
    zip_dict = dict()
    none_resp = "".encode("utf-8")

    @lru_cache(maxsize=10000)
    def _get_file(self, file):
        """
        Retrieves the file data from zip files.

        Args:
            file (str): Image filename.

        Returns:
            bytes: Image data.
        """
        zf = self.zip_dict.get(file)
        if zf is None:
            logging.warning(f"File {file} not found.")
            return self.none_resp
        zf = self.zip_data[zf]
        if zf is None:
            logging.warning(f"File {file} not found.")
            return self.none_resp
        file = zf.read(file)
        return file

    def do_GET(self):
        """
        Handles GET requests.

        Returns:
            None
        """
        if self.path == "/all-files":
            self.send_response(200)
            self.end_headers()
            names = list(self.zip_dict.keys())
            self.wfile.write("\n".join(names).encode("utf-8"))
        else:
            self.send_response(200)
            self.end_headers()
            params = parse_params(self.path)
            file = params.get("file")
            resp = self.none_resp
            if file is not None:
                resp = self._get_file(file)
            self.wfile.write(resp)

    def log_request(self, format, *args):
        """
        Logs the HTTP request.

        Returns:
            None
        """
        return

    def log_message(self, format, *args):
        """
        Logs the message.

        Returns:
            None
        """
        return


@register_service("core/service/zip_files")
class ZipFilesService(GenericService):
    def __init__(self, config: CoreConfigureParser):
        self.config = config
        config.set_default_section("core/service/zip_files")
        self.ip = config.getoption("ip", "0.0.0.0")
        self.port = config.getoption("port", 11230)
        self.name = config.getoption("processname", "core_zip_files_service")
        self.zip_folder = config.getoption("zip_folder", None)
        self.zip_extension = config.getoption("zip_extension", ".zip")
        self.num_thread = config.getoption("num_thread", 20)
        assert self.zip_folder is not None

        if isinstance(self.zip_folder, str):
            self.zip_folder = os.path.abspath(self.zip_folder)
            zip_files = [
                os.path.join(self.zip_folder, f)
                for f in os.listdir(self.zip_folder)
                if f.endswith(self.zip_extension)
            ]
        elif isinstance(self.zip_folder, list):
            self.zip_folder = [os.path.abspath(f) for f in self.zip_folder]
            zip_files = []
            for folder in list(set(self.zip_folder)):
                zip_files += [
                    os.path.join(folder, f)
                    for f in os.listdir(folder)
                    if f.endswith(self.zip_extension)
                ]

        self.zip_data = get_zipfiles(zip_files, self.num_thread)
        self.zip_dict = {
            v: i for i, k in enumerate(self.zip_data) for v in k.namelist()
        }

    def start(self, **kwargs):
        ZipFilesServer.zip_data = self.zip_data
        ZipFilesServer.zip_dict = self.zip_dict
        self.httpd = http.server.HTTPServer((self.ip, self.port), ZipFilesServer)
        self.httpd.serve_forever()

    def stop(self, **kwargs):
        pass

    def restart(self, **kwargs):
        pass



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli import hf_endpoint_url

# pretrained infos
pretrained_roberta_infos = {
    "roberta-base": {
        "config": hf_endpoint_url("/roberta-base/resolve/main/config.json"),
        "vocab": hf_endpoint_url("/roberta-base/resolve/main/vocab.json"),
        "merge": hf_endpoint_url("/roberta-base/resolve/main/merges.txt"),
        "weight": hf_endpoint_url("/roberta-base/resolve/main/pytorch_model.bin"),
    },
    "roberta-large": {
        "config": hf_endpoint_url("/roberta-large/resolve/main/config.json"),
        "vocab": hf_endpoint_url("/roberta-large/resolve/main/vocab.json"),
        "merge": hf_endpoint_url("/roberta-large/resolve/main/merges.txt"),
        "weight": hf_endpoint_url("/roberta-large/resolve/main/pytorch_model.bin"),
    },
}

import unitorch.cli.models.roberta.modeling
import unitorch.cli.models.roberta.processing
from unitorch.cli.models.roberta.modeling import RobertaForClassification
from unitorch.cli.models.roberta.processing import RobertaProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import numpy as np
from PIL import Image, ImageFilter
from typing import Any, Dict, List, Optional, Tuple, Union
from unitorch.utils import is_opencv_available


def canny(image: Image.Image):
    if is_opencv_available():
        import cv2

        image = np.array(image, np.uint8)
        image = cv2.cvtColor(image, cv2.COLOR_RGB2GRAY)
        image = cv2.Canny(image, 100, 200)
        image = Image.fromarray(image)
    else:
        image = image.convert("L")
        image = image.filter(ImageFilter.FIND_EDGES)
    return image



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import numpy as np
from PIL import Image, ImageFilter
from typing import Any, Dict, List, Optional, Tuple, Union
from unitorch.utils import is_opencv_available
from unitorch.cli import CoreConfigureParser

from unitorch.cli.pipelines.stable_flux.text2image import (
    StableFluxForText2ImageGenerationPipeline,
)
from unitorch.cli.pipelines.stable_flux.image2image import (
    StableFluxForImage2ImageGenerationPipeline,
)
from unitorch.cli.pipelines.stable_flux.image_control import (
    StableFluxForImageControlGenerationPipeline,
)
from unitorch.cli.pipelines.stable_flux.image_redux import (
    StableFluxForImageReduxGenerationPipeline,
)
from unitorch.cli.pipelines.stable_flux.inpainting import (
    StableFluxForImageInpaintingPipeline,
)
from unitorch.cli.pipelines.stable_flux.redux_inpainting import (
    StableFluxForReduxInpaintingPipeline,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import numpy as np
from PIL import Image, ImageFilter
from typing import Any, Dict, List, Optional, Tuple, Union
from unitorch.utils import is_opencv_available

from unitorch.cli.pipelines.stable.text2image import (
    StableForText2ImageGenerationPipeline,
)
from unitorch.cli.pipelines.stable.image2image import (
    StableForImage2ImageGenerationPipeline,
)
from unitorch.cli.pipelines.stable.inpainting import StableForImageInpaintingPipeline
from unitorch.cli.pipelines.stable.resolution import StableForImageResolutionPipeline
from unitorch.cli.pipelines.stable.image2video import (
    StableForImage2VideoGenerationPipeline,
)
from unitorch.cli.pipelines.stable.interrogator import ClipInterrogatorPipeline



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli import hf_endpoint_url

# pretrained infos
pretrained_dpt_infos = {
    "dpt-large": {
        "config": hf_endpoint_url("/Intel/dpt-large/resolve/main/config.json"),
        "vision_config": hf_endpoint_url(
            "/Intel/dpt-large/resolve/main/preprocessor_config.json"
        ),
        "weight": hf_endpoint_url("/Intel/dpt-large/resolve/main/pytorch_model.bin"),
    },
}

import unitorch.cli.models.dpt.modeling
import unitorch.cli.models.dpt.processing
from unitorch.cli.models.dpt.modeling import DPTForDepthEstimation
from unitorch.cli.models.dpt.processing import DPTProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import io
import re
import gc
import json
import logging
import torch
import hashlib
import asyncio
import pandas as pd
from PIL import Image
from torch import autocast
from fastapi import APIRouter, UploadFile, File
from fastapi.responses import StreamingResponse
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from diffusers.utils import numpy_to_pil
from diffusers.models import ControlNetModel
from diffusers.pipelines import (
    StableDiffusionPipeline,
    StableDiffusionImg2ImgPipeline,
    StableDiffusionInpaintPipeline,
    StableDiffusionUpscalePipeline,
    StableDiffusionDepth2ImgPipeline,
    StableVideoDiffusionPipeline,
    StableDiffusionControlNetPipeline,
    StableDiffusionControlNetImg2ImgPipeline,
    StableDiffusionControlNetInpaintPipeline,
)
from unitorch import is_xformers_available
from unitorch.utils import is_remote_url
from unitorch.models.diffusers import StableForImage2VideoGeneration
from unitorch.models.diffusers import StableVideoProcessor
from unitorch.models.diffusers.modeling_stable import StableVideoDiffusionPipelineV2

from unitorch.utils import pop_value, nested_dict_value, tensor2vid
from unitorch.cli import (
    cached_path,
    register_fastapi,
    add_default_section_for_init,
    add_default_section_for_function,
)
from unitorch.cli import CoreConfigureParser, GenericFastAPI
from unitorch.cli.models.diffusers import (
    pretrained_stable_infos,
    pretrained_stable_extensions_infos,
    load_weight,
)
from unitorch.cli.pipelines import Schedulers
from unitorch.cli.models.diffusion_utils import export_to_video


class StableForImage2VideoFastAPIPipeline(StableForImage2VideoGeneration):
    def __init__(
        self,
        config_path: str,
        image_config_path: str,
        image_process_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        weight_path: Optional[Union[str, List[str]]] = None,
        state_dict: Optional[Dict[str, Any]] = None,
        device: Optional[Union[str, int]] = "cpu",
        enable_cpu_offload: Optional[bool] = False,
        enable_xformers: Optional[bool] = False,
    ):
        super().__init__(
            config_path=config_path,
            image_config_path=image_config_path,
            image_process_config_path=image_process_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
        )
        self.processor = StableVideoProcessor(
            vision_config_path=image_process_config_path,
            vae_config_path=vae_config_path,
        )
        self._device = "cpu" if device == "cpu" else int(device)

        self.from_pretrained(weight_path, state_dict=state_dict)
        self.eval()

        self._enable_cpu_offload = enable_cpu_offload
        self._enable_xformers = enable_xformers

        self.pipeline = StableVideoDiffusionPipelineV2(
            vae=self.vae,
            image_encoder=self.image,
            unet=self.unet,
            scheduler=self.scheduler,
            feature_extractor=self.image_processor,
        )
        self.pipeline.set_progress_bar_config(disable=True)
        if self._enable_cpu_offload and self._device != "cpu":
            self.pipeline.enable_model_cpu_offload(self._device)
        else:
            self.to(device=self._device)

        if self._enable_xformers and self._device != "cpu":
            assert is_xformers_available(), "Please install xformers first."
            self.pipeline.enable_xformers_memory_efficient_attention()

    @classmethod
    @add_default_section_for_init("core/fastapi/pipeline/stable/image2video")
    def from_core_configure(
        cls,
        config,
        pretrained_name: Optional[str] = None,
        config_path: Optional[str] = None,
        image_config_path: Optional[str] = None,
        vae_config_path: Optional[str] = None,
        scheduler_config_path: Optional[str] = None,
        image_process_config_path: Optional[str] = None,
        quant_config_path: Optional[str] = None,
        pretrained_weight_path: Optional[str] = None,
        device: Optional[str] = None,
        **kwargs,
    ):
        config.set_default_section("core/fastapi/pipeline/stable/image2video")
        pretrained_name = pretrained_name or config.getoption(
            "pretrained_name", "stable-video-diffusion-img2vid-xt-1-1"
        )
        pretrained_infos = nested_dict_value(pretrained_stable_infos, pretrained_name)

        config_path = config.getoption("config_path", config_path)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_infos, "unet", "config"),
        )
        config_path = cached_path(config_path)

        image_config_path = config.getoption("image_config_path", image_config_path)
        image_config_path = pop_value(
            image_config_path,
            nested_dict_value(pretrained_infos, "image", "config"),
        )
        image_config_path = cached_path(image_config_path)

        vae_config_path = config.getoption("vae_config_path", vae_config_path)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrained_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)

        scheduler_config_path = config.getoption(
            "scheduler_config_path", scheduler_config_path
        )
        scheduler_config_path = pop_value(
            scheduler_config_path,
            nested_dict_value(pretrained_infos, "scheduler"),
        )
        scheduler_config_path = cached_path(scheduler_config_path)

        image_process_config_path = config.getoption(
            "image_process_config_path", image_process_config_path
        )
        image_process_config_path = pop_value(
            image_process_config_path,
            nested_dict_value(pretrained_infos, "image", "vision_config"),
        )
        image_process_config_path = cached_path(image_process_config_path)

        quant_config_path = config.getoption("quant_config_path", quant_config_path)
        if quant_config_path is not None:
            quant_config_path = cached_path(quant_config_path)

        weight_path = config.getoption("pretrained_weight_path", pretrained_weight_path)
        device = config.getoption("device", "cpu") if device is None else device
        enable_cpu_offload = config.getoption("enable_cpu_offload", False)
        enable_xformers = config.getoption("enable_xformers", False)

        state_dict = None
        if weight_path is None and pretrained_infos is not None:
            state_dict = [
                load_weight(nested_dict_value(pretrained_infos, "unet", "weight")),
                load_weight(nested_dict_value(pretrained_infos, "image", "weight")),
                load_weight(nested_dict_value(pretrained_infos, "vae", "weight")),
            ]

        inst = cls(
            config_path=config_path,
            image_config_path=image_config_path,
            image_process_config_path=image_process_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            weight_path=weight_path,
            state_dict=state_dict,
            device=device,
            enable_cpu_offload=enable_cpu_offload,
            enable_xformers=enable_xformers,
        )
        return inst

    @torch.no_grad()
    @autocast(device_type=("cuda" if torch.cuda.is_available() else "cpu"))
    @add_default_section_for_function("core/fastapi/pipeline/stable/image2video")
    def __call__(
        self,
        image: Image.Image,
        height: Optional[int] = 576,
        width: Optional[int] = 1024,
        num_frames: Optional[int] = 30,
        num_fps: Optional[int] = 6,
        min_guidance_scale: Optional[float] = 1.0,
        max_guidance_scale: Optional[float] = 2.5,
        motion_bucket_id: Optional[int] = 127,
        decode_chunk_size: Optional[int] = 8,
        num_timesteps: Optional[int] = 50,
        seed: Optional[int] = 1123,
    ):
        image = image.convert("RGB")
        inputs = self.processor.image2video_inputs(
            image,
            vae_image=image.resize((width, height), resample=Image.LANCZOS),
        )

        self.seed = seed

        inputs = {k: v.unsqueeze(0) if v is not None else v for k, v in inputs.items()}
        inputs = {
            k: v.to(device=self.device) if v is not None else v
            for k, v in inputs.items()
        }

        outputs = self.pipeline(
            image=inputs["pixel_values"],
            vae_image=inputs["vae_pixel_values"],
            height=height,
            width=width,
            num_frames=num_frames,
            min_guidance_scale=min_guidance_scale,
            max_guidance_scale=max_guidance_scale,
            fps=num_fps,
            motion_bucket_id=motion_bucket_id,
            decode_chunk_size=decode_chunk_size,
            generator=torch.Generator(device=self.pipeline.device).manual_seed(
                self.seed
            ),
            num_inference_steps=num_timesteps,
            output_type="pt",
        )

        frames = tensor2vid(outputs.frames)
        name = export_to_video(frames)
        return name


@register_fastapi("core/fastapi/stable/image2video")
class StableImage2VideoFastAPI(GenericFastAPI):
    def __init__(self, config: CoreConfigureParser):
        self.config = config
        config.set_default_section(f"core/fastapi/stable/image2video")
        router = config.getoption("router", "/core/fastapi/stable/image2video")
        self._pipe = None
        self._router = APIRouter(prefix=router)
        self._router.add_api_route("/generate", self.serve, methods=["POST"])
        self._router.add_api_route("/status", self.status, methods=["GET"])
        self._router.add_api_route("/start", self.start, methods=["POST"])
        self._router.add_api_route("/stop", self.stop, methods=["GET"])
        self._lock = asyncio.Lock()

    @property
    def router(self):
        return self._router

    def start(
        self,
        pretrained_name: Optional[str] = "stable-video-diffusion-img2vid-xt-1-1",
        pretrained_lora_names: Optional[Union[str, List[str]]] = None,
        pretrained_lora_weights: Optional[Union[float, List[float]]] = 1.0,
        pretrained_lora_alphas: Optional[Union[float, List[float]]] = 32.0,
    ):
        self._pipe = StableForImage2VideoFastAPIPipeline.from_core_configure(
            self.config,
            pretrained_name=pretrained_name,
            pretrained_lora_names=pretrained_lora_names,
            pretrained_lora_weights=pretrained_lora_weights,
            pretrained_lora_alphas=pretrained_lora_alphas,
        )
        return "start success"

    def stop(self):
        self._pipe.to("cpu")
        del self._pipe
        gc.collect()
        torch.cuda.empty_cache()
        self._pipe = None
        return "stop success"

    def status(self):
        return "running" if self._pipe is not None else "stopped"

    async def serve(
        self,
        image: UploadFile,
        num_frames: Optional[int] = 30,
        num_fps: Optional[int] = 6,
        min_guidance_scale: Optional[float] = 1.0,
        max_guidance_scale: Optional[float] = 2.5,
        motion_bucket_id: Optional[int] = 127,
        decode_chunk_size: Optional[int] = 8,
        num_timesteps: Optional[int] = 50,
        seed: Optional[int] = 1123,
    ):
        assert self._pipe is not None
        image_bytes = await image.read()
        image = Image.open(io.BytesIO(image_bytes))
        async with self._lock:
            video = self._pipe(
                image,
                num_frames=num_frames,
                num_fps=num_fps,
                min_guidance_scale=min_guidance_scale,
                max_guidance_scale=max_guidance_scale,
                motion_bucket_id=motion_bucket_id,
                decode_chunk_size=decode_chunk_size,
                num_timesteps=num_timesteps,
                seed=seed,
            )
        buffer = io.BytesIO()
        with open(video, "rb") as f:
            buffer.write(f.read())
        buffer.seek(0)
        return StreamingResponse(
            io.BytesIO(buffer.getvalue()),
            media_type="video/mp4",
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import cv2
import torch
import json
import numpy as np
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers import CLIPTokenizer, T5Tokenizer, CLIPImageProcessor
from torchvision.transforms import (
    Resize,
    CenterCrop,
    RandomCrop,
    Lambda,
    ToTensor,
    Normalize,
    Compose,
    RandomHorizontalFlip,
)
from diffusers.image_processor import VaeImageProcessor
from unitorch.models import (
    HfTextClassificationProcessor,
    HfImageClassificationProcessor,
    GenericOutputs,
)


class WanProcessor(HfTextClassificationProcessor):
    def __init__(
        self,
        vocab_path: str,
        vae_config_path: Optional[str] = None,
        image_config_path: Optional[str] = None,
        max_seq_length: Optional[int] = 77,
        position_start_id: Optional[int] = 0,
        video_size: Optional[Tuple[int, int]] = None,
    ):
        tokenizer = T5Tokenizer(
            vocab_file=vocab_path,
        )

        tokenizer.bos_token_id = 0
        tokenizer.cls_token = tokenizer.convert_ids_to_tokens(0)
        tokenizer.sep_token = tokenizer.eos_token
        tokenizer.sep_token_id = tokenizer.eos_token_id

        HfTextClassificationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            position_start_id=position_start_id,
        )

        if video_size is not None:
            self.video_size = (
                video_size
                if isinstance(video_size, tuple)
                else (video_size, video_size)
            )
        else:
            self.video_size = None

        self.divisor = 16
        if self.video_size is not None:
            self.video_size = (
                self.video_size[0] // self.divisor * self.divisor,
                self.video_size[1] // self.divisor * self.divisor,
            )
            self.center_crop_processor = CenterCrop(
                size=(self.video_size[1], self.video_size[0])
            )
        else:
            self.center_crop_processor = None
        self.frame_processor = Compose(
            [
                ToTensor(),
                Normalize(mean=[0.5, 0.5, 0.5], std=[0.5, 0.5, 0.5]),
            ]
        )

        if vae_config_path is not None:
            vae_config_dict = json.load(open(vae_config_path))
            vae_scale_factor = 2 ** (
                len(vae_config_dict.get("block_out_channels", [])) - 1
            )
            self.vae_image_processor = VaeImageProcessor(
                vae_scale_factor=vae_scale_factor
            )
        else:
            self.vae_image_processor = None

        if image_config_path is not None:
            self.vision_processor = CLIPImageProcessor.from_json_file(image_config_path)
        else:
            self.vision_processor = None

    def get_video_frames(self, video: Union[cv2.VideoCapture, str]):
        if isinstance(video, str):
            video = cv2.VideoCapture(video)

        if isinstance(video, cv2.VideoCapture):
            frames = []
            while video.isOpened():
                ret, frame = video.read()
                if not ret:
                    break
                rgb = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
                pil_img = Image.fromarray(rgb)
                frames.append(pil_img)
        else:
            frames = video

        return frames

    def text2video(
        self,
        prompt: str,
        video: Union[cv2.VideoCapture, str, List[Image.Image]],
        max_seq_length: Optional[int] = None,
    ):
        frames = self.get_video_frames(video)

        pixel_values = []
        for frame in frames:
            if self.frame_processor is not None:
                if self.video_size is not None:
                    width, height = frame.size
                    scale = max(self.video_size[0] / width, self.video_size[1] / height)
                    frame = frame.resize(
                        (round(width * scale), round(height * scale)),
                        resample=Image.LANCZOS,
                    )
                    frame = self.center_crop_processor(frame)
                else:
                    width, height = frame.size
                    new_width = width // self.divisor * self.divisor
                    new_height = height // self.divisor * self.divisor
                    frame = frame.resize(
                        (new_width, new_height), resample=Image.LANCZOS
                    )
                pixel_frame = self.frame_processor(frame)
                pixel_values.append(pixel_frame)
            else:
                raise ValueError(
                    "frame_processor is None, please set video_size to process video"
                )

        pixel_values = torch.stack(pixel_values, dim=0)
        pixel_values = pixel_values.permute(1, 0, 2, 3)

        prompt_outputs = self.classification(prompt, max_seq_length=max_seq_length)

        return GenericOutputs(
            pixel_values=pixel_values,
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
        )

    def image2video(
        self,
        prompt: str,
        video: Union[cv2.VideoCapture, str, List[Image.Image]],
        image: Optional[Union[Image.Image, str]] = None,
        max_seq_length: Optional[int] = None,
    ):
        frames = self.get_video_frames(video)

        outputs = self.text2video(
            prompt=prompt,
            video=frames,
            max_seq_length=max_seq_length,
        )
        if isinstance(image, str):
            image = Image.open(image).convert("RGB")

        if image is None:
            image = frames[0].convert("RGB")

        if self.video_size is not None:
            width, height = image.size
            scale = max(self.video_size[0] / width, self.video_size[1] / height)
            image = image.resize(
                (round(height * scale), round(width * scale)), resample=Image.LANCZOS
            )
            image = self.center_crop_processor(image)
        else:
            width, height = image.size
            new_width = width // self.divisor * self.divisor
            new_height = height // self.divisor * self.divisor
            image = image.resize((new_width, new_height), resample=Image.LANCZOS)

        condition_pixel_values = self.vision_processor.preprocess(
            image, return_tensors="pt"
        ).pixel_values[0]

        vae_pixel_values = self.vae_image_processor.preprocess(image)[0]

        return GenericOutputs(
            pixel_values=outputs.pixel_values,
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            condition_pixel_values=condition_pixel_values,
            vae_pixel_values=vae_pixel_values,
        )

    def text2video_inputs(
        self,
        prompt: str,
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        prompt_outputs = self.classification(prompt, max_seq_length=max_seq_length)
        negative_prompt_outputs = self.classification(
            negative_prompt, max_seq_length=max_seq_length
        )

        return GenericOutputs(
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
            negative_input_ids=negative_prompt_outputs.input_ids,
            negative_attention_mask=negative_prompt_outputs.attention_mask,
        )

    def image2video_inputs(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        if isinstance(image, str):
            image = Image.open(image)
        image = image.convert("RGB")

        if self.video_size is not None:
            width, height = image.size
            scale = max(self.video_size[0] / width, self.video_size[1] / height)
            image = image.resize(
                (round(height * scale), round(width * scale)), resample=Image.LANCZOS
            )
            image = self.center_crop_processor(image)
        else:
            width, height = image.size
            new_width = width // self.divisor * self.divisor
            new_height = height // self.divisor * self.divisor
            image = image.resize((new_width, new_height), resample=Image.LANCZOS)

        vae_pixel_values = self.vae_image_processor.preprocess(image)[0]
        pixel_values = self.vision_processor.preprocess(
            image, return_tensors="pt"
        ).pixel_values[0]
        text_outputs = self.text2video_inputs(
            prompt=prompt,
            negative_prompt=negative_prompt,
            max_seq_length=max_seq_length,
        )

        return GenericOutputs(
            input_ids=text_outputs.input_ids,
            attention_mask=text_outputs.attention_mask,
            negative_input_ids=text_outputs.negative_input_ids,
            negative_attention_mask=text_outputs.negative_attention_mask,
            condition_pixel_values=pixel_values,
            vae_pixel_values=vae_pixel_values,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.segformer.modeling import SegformerForSegmentation
from unitorch.models.segformer.processing import SegformerProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import torch
import numpy as np
from PIL import Image
from torchvision.transforms import Resize, CenterCrop, ToTensor, Normalize, Compose
from transformers import ViTImageProcessor
from transformers.image_utils import to_numpy_array, ChannelDimension
from transformers.image_transforms import to_channel_dimension_format
from unitorch.utils import pop_value
from unitorch.models import HfImageClassificationProcessor, GenericOutputs


class SwinProcessor(HfImageClassificationProcessor):
    def __init__(
        self,
        vision_config_path: str,
    ):
        """
        Initializes a SwinProcessor for image classification tasks.

        Args:
            vision_config_path (str): The path to the ViTImageProcessor configuration file.
        """
        vision_processor = ViTImageProcessor.from_json_file(vision_config_path)
        super().__init__(
            vision_processor=vision_processor,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import json
import random
import logging
import torch
import torch.nn.functional as F
import torch.nn as nn
from torch import autocast
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers.models.segformer.modeling_segformer import (
    SegformerConfig,
    SegformerModel,
    SegformerDecodeHead,
)
from unitorch.models import GenericModel, GenericOutputs


class SegformerForSegmentation(GenericModel):
    def __init__(
        self,
        config_path: str,
    ):
        """
        Initializes a SamForSegmentation model for segmentation tasks.

        Args:
            config_path (str): The path to the Sam Transformer configuration file.
        """
        super().__init__()
        config = SegformerConfig.from_json_file(config_path)

        self.segformer = SegformerModel(config)
        self.decode_head = SegformerDecodeHead(config)

        self.init_weights()

    def forward(
        self,
        pixel_values: torch.Tensor,
    ):
        """
        Performs a forward pass of the SamForSegmentation model.
        """
        outputs = self.segformer(
            pixel_values=pixel_values,
            output_hidden_states=True,
            return_dict=True,
        )

        logits = self.decode_head(outputs.hidden_states)

        upsampled_logits = nn.functional.interpolate(
            logits,
            scale_factor=4,
            mode="bilinear",
            align_corners=False,
        )

        return GenericOutputs(logits=upsampled_logits)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli.fastapis.controlnet.text2image import (
    ControlNetForText2ImageFastAPIPipeline,
)
from unitorch.cli.fastapis.controlnet.image2image import (
    ControlNetForImage2ImageFastAPIPipeline,
)
from unitorch.cli.fastapis.controlnet.inpainting import (
    ControlNetForImageInpaintingFastAPIPipeline,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.peft.diffusers.modeling_stable import (
    StableLoraForText2ImageGeneration,
    StableLoraForImageInpainting,
)
from unitorch.models.peft.diffusers.modeling_stable_xl import (
    StableXLLoraForText2ImageGeneration,
    StableXLLoraForImageInpainting,
)
from unitorch.models.peft.diffusers.modeling_stable_3 import (
    Stable3LoraForText2ImageGeneration,
    Stable3LoraForImageInpainting,
)
from unitorch.models.peft.diffusers.modeling_stable_flux import (
    StableFluxLoraForText2ImageGeneration,
    StableFluxLoraForImageInpainting,
)
from unitorch.models.peft.diffusers.modeling_controlnet import (
    ControlNetLoraForText2ImageGeneration,
)
from unitorch.models.peft.diffusers.modeling_controlnet_xl import (
    ControlNetXLLoraForText2ImageGeneration,
)
from unitorch.models.peft.diffusers.modeling_controlnet_3 import (
    ControlNet3LoraForText2ImageGeneration,
)
from unitorch.models.peft.diffusers.modeling_controlnet_flux import (
    ControlNetFluxLoraForText2ImageGeneration,
)
from unitorch.models.peft.diffusers.modeling_adapter import (
    StableAdapterLoraForText2ImageGeneration,
)
from unitorch.models.peft.diffusers.modeling_adapter_xl import (
    StableXLAdapterLoraForText2ImageGeneration,
)
from unitorch.models.peft.diffusers.modeling_wan import (
    WanLoraForText2VideoGeneration,
    WanLoraForImage2VideoGeneration,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.bart.modeling import BartForGeneration
from unitorch.models.bart.processing import BartProcessor, get_bart_tokenizer



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

""" Wrapper for ngram_repeat_block cuda extension """
import torch.nn as nn
from torch.autograd import Function

try:
    import unitorch.clib.ngram_repeat_block_cuda as ngram_repeat_block_cuda

    is_kernel_available = True
except:
    is_kernel_available = False


class NGramRepeatBlockFunction(Function):
    def forward(
        self,
        tokens,
        lprobs,
        bsz,
        step,
        beam_size,
        no_repeat_ngram_size,
    ):
        outputs = ngram_repeat_block_cuda.forward(
            tokens, lprobs, bsz, step, beam_size, no_repeat_ngram_size
        )
        return outputs

    def backward(*args):
        raise NotImplementedError


class NGramRepeatBlock(nn.Module):
    def __init__(self):
        super(NGramRepeatBlock, self).__init__()

    def reset_parameters(self):
        pass

    @classmethod
    def is_available(cls):
        return is_kernel_available

    def forward(
        self,
        tokens,
        lprobs,
        bsz,
        step,
        beam_size,
        no_repeat_ngram_size,
    ):
        assert tokens.size(0) == bsz * beam_size
        assert lprobs.size(0) == bsz * beam_size

        return NGramRepeatBlockFunction.apply(
            tokens, lprobs, bsz, step, beam_size, no_repeat_ngram_size
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import json
import random
import logging
import torch
import torch.nn.functional as F
import torch.nn as nn
from torch import autocast
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers.models.sam.modeling_sam import SamConfig, SamModel
from transformers.models.sam.image_processing_sam import _build_point_grid
from unitorch.models import GenericModel, GenericOutputs
from unitorch.models.peft import PeftWeightLoaderMixin


class SamForSegmentation(GenericModel, PeftWeightLoaderMixin):
    prefix_keys_in_state_dict = {
        "^mask_decoder.*": "sam.",
        "^vision_encoder.*": "sam.",
        "^prompt_encoder.*": "sam.",
        "^shared_image_embedding.*": "sam.",
    }

    replace_keys_in_peft_state_dict = {"peft_model.base_model.model.": ""}

    def __init__(
        self,
        config_path: str,
    ):
        """
        Initializes a SamForSegmentation model for segmentation tasks.

        Args:
            config_path (str): The path to the Sam Transformer configuration file.
        """
        super().__init__()
        config = SamConfig.from_json_file(config_path)

        self.sam = SamModel(config)
        self.init_weights()

    def forward(self):
        """
        Performs a forward pass of the SamForSegmentation model.
        """
        raise NotImplementedError

    def segment(
        self,
        pixel_values: torch.Tensor,
        input_points: torch.Tensor,
        input_labels: Optional[torch.Tensor] = None,
        input_boxes: Optional[torch.Tensor] = None,
        input_masks: Optional[torch.Tensor] = None,
    ):
        """
        Performs a forward pass of the SamForSegmentation model.
        """
        outputs = self.sam(
            pixel_values,
            input_points=input_points,
            input_labels=input_labels,
            # input_boxes=input_boxes,
            input_masks=input_masks,
        )
        return GenericOutputs(masks=outputs.pred_masks, scores=outputs.iou_scores)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.mbart import MBartProcessor as _MBartProcessor
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli import WriterOutputs
from unitorch.cli.models import (
    TensorsInputs,
    GenerationOutputs,
    GenerationTargets,
)
from unitorch.cli.models.mbart import pretrained_mbart_infos


class MBartProcessor(_MBartProcessor):
    """Processor for the MBart model."""

    def __init__(
        self,
        vocab_path: str,
        max_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 48,
        special_input_ids: Optional[Dict] = dict(),
    ):
        """
        Initialize the MBartProcessor.

        Args:
            vocab_path (str): The path to the vocabulary file.
            max_seq_length (int, optional): The maximum input sequence length. Defaults to 128.
            max_gen_seq_length (int, optional): The maximum generated sequence length. Defaults to 48.
            special_input_ids (Dict, optional): Special input IDs. Defaults to an empty dictionary.
        """
        super().__init__(
            vocab_path=vocab_path,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
            special_input_ids=special_input_ids,
        )

    @classmethod
    @add_default_section_for_init("core/process/mbart")
    def from_core_configure(cls, config, **kwargs):
        """
        Create an instance of MBartProcessor from a core configuration.

        Args:
            config: The core configuration.
            **kwargs: Additional keyword arguments.

        Returns:
            MBartProcessor: An instance of MBartProcessor.
        """
        config.set_default_section("core/process/mbart")
        pretrained_name = config.getoption("pretrained_name", "mbart-large-cc25")
        vocab_path = config.getoption("vocab_path", None)
        vocab_path = pop_value(
            vocab_path,
            nested_dict_value(pretrained_mbart_infos, pretrained_name, "vocab"),
        )
        vocab_path = cached_path(vocab_path)

        return {
            "vocab_path": vocab_path,
        }

    @register_process("core/process/mbart/generation/inputs")
    def _generation_inputs(
        self,
        text: str,
        max_seq_length: Optional[int] = None,
    ):
        """
        Preprocess the input text for generation tasks.

        Args:
            text (str): The input text.
            max_seq_length (int, optional): The maximum sequence length. Defaults to None.

        Returns:
            TensorsInputs: The processed input tensors.
        """
        outputs = super().generation_inputs(
            text=text,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(input_ids=outputs.input_ids)

    @register_process("core/process/mbart/generation/labels")
    def _generation_labels(
        self,
        text: str,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Preprocess the target text for generation tasks.

        Args:
            text (str): The target text.
            max_gen_seq_length (int, optional): The maximum generation sequence length. Defaults to None.

        Returns:
            GenerationTargets: The processed generation targets.
        """
        outputs = super().generation_labels(
            text=text,
            max_gen_seq_length=max_gen_seq_length,
        )
        return GenerationTargets(
            refs=outputs.input_ids,
            masks=outputs.attention_mask,
        )

    @register_process("core/process/mbart/generation")
    def _generation(
        self,
        text: str,
        text_pair: Optional[str] = None,
        max_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Preprocess the input and target texts for generation tasks.

        Args:
            text (str): The input text.
            text_pair (str, optional): The paired input text. Defaults to None.
            max_seq_length (int, optional): The maximum sequence length. Defaults to None.
            max_gen_seq_length (int, optional): The maximum generation sequence length. Defaults to None.

        Returns:
            Tuple[TensorsInputs, GenerationTargets]: The processed input tensors and generation targets.
        """
        outputs = super().generation(
            text=text,
            text_pair=text_pair,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )
        return TensorsInputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            decoder_input_ids=outputs.input_ids_pair,
            decoder_attention_mask=outputs.attention_mask_pair,
        ), GenerationTargets(
            refs=outputs.input_ids_label,
            masks=outputs.attention_mask_label,
        )

    @register_process("core/postprocess/mbart/detokenize")
    def _detokenize(
        self,
        outputs: GenerationOutputs,
    ):
        """
        Detokenize the generated sequences.

        Args:
            outputs (GenerationOutputs): The generation outputs.

        Returns:
            WriterOutputs: The detokenized writer outputs.
        """
        results = outputs.to_pandas()
        assert results.shape[0] == 0 or results.shape[0] == outputs.sequences.shape[0]

        decoded = super().detokenize(sequences=outputs.sequences)
        results["decoded"] = decoded
        return WriterOutputs(results)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import io
import logging
import json
import torch
import logging
import torch.nn as nn
import torch.nn.functional as F
import transformers
import torch.distributed as dist
from torch import autocast
from transformers.utils import is_remote_url
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from megatron.core import mpu
from megatron.core.transformer import TransformerConfig
from megatron.core.models.gpt import GPTModel
from megatron.core.pipeline_parallel.schedules import get_forward_backward_func
from megatron.core.models.gpt.gpt_layer_specs import (
    get_gpt_decoder_block_spec,
    get_gpt_layer_local_spec,
    get_gpt_layer_with_transformer_engine_spec,
    get_gpt_mtp_block_spec,
)
from megatron.core.transformer.enums import ModelType
from unitorch import hf_cached_path
from unitorch.utils import read_json_file
from unitorch.utils.decorators import replace
from unitorch.models import GenericModel, GenericOutputs
from unitorch.models.megatron import GenericMegatronModel


class MegatronGPTForGeneration(GenericMegatronModel):
    """
    Megatron GPT Model
    """

    modules_to_save_checkpoints = ["model"]

    def __init__(
        self,
        config_path: str,
        vocab_size: int,
        max_position_embeddings: Optional[int] = 512,
        normalization: Optional[str] = "RMSNorm",
        position_embedding_type: Optional[str] = "rope",
        rotary_percent: Optional[float] = 0.5,
        rotary_base: Optional[int] = 10000,
        rope_scaling: Optional[bool] = False,
        num_experts: Optional[int] = None,
        use_transformer_engine: Optional[bool] = False,
    ):
        super().__init__()
        self.config = TransformerConfig(
            **read_json_file(config_path),
            tensor_model_parallel_size=mpu.get_tensor_model_parallel_world_size(),
            pipeline_model_parallel_size=mpu.get_pipeline_model_parallel_world_size(),
            virtual_pipeline_model_parallel_size=mpu.get_virtual_pipeline_model_parallel_world_size(),
            context_parallel_size=mpu.get_context_parallel_world_size(),
            pipeline_dtype=torch.bfloat16,
            bf16=True,
            autocast_dtype=torch.bfloat16,
        )
        self.use_transformer_engine = use_transformer_engine
        self.tp_size = mpu.get_tensor_model_parallel_world_size()
        self.is_pp_first_rank = mpu.is_pipeline_first_stage(ignore_virtual=True)
        self.is_pp_last_rank = mpu.is_pipeline_last_stage(ignore_virtual=True)
        use_preprocess, use_postprocess = self.is_pp_first_rank, self.is_pp_last_rank

        if num_experts is not None:
            if self.use_transformer_engine:
                self.layer_spec = get_gpt_layer_with_transformer_engine_spec(
                    num_experts=num_experts,
                    moe_grouped_gemm=True,
                    qk_layernorm=True,
                )
            else:
                self.layer_spec = get_gpt_layer_local_spec(
                    num_experts=num_experts,
                    moe_grouped_gemm=True,
                    qk_layernorm=True,
                    normalization=normalization,
                )
        self.model = GPTModel(
            config=self.config,
            transformer_layer_spec=get_gpt_decoder_block_spec(
                self.config,
                use_transformer_engine=use_transformer_engine,
                normalization=normalization,
            ),
            pre_process=use_preprocess,
            post_process=use_postprocess,
            vocab_size=vocab_size,
            max_sequence_length=max_position_embeddings,
            position_embedding_type=position_embedding_type,
            rotary_percent=rotary_percent,
            rotary_base=rotary_base,
            rope_scaling=rope_scaling,
            mtp_block_spec=get_gpt_mtp_block_spec(
                self.config,
                self.layer_spec,
                use_transformer_engine=use_transformer_engine,
            )
            if num_experts is not None
            else None,
        )
        self.model_type = self.model.model_type
        self.model = self.model.to(torch.bfloat16)

    def set_input_tensor(self, input_tensor):
        self.model.decoder.set_input_tensor(input_tensor[0])

    def forward(
        self,
        input_ids: torch.Tensor = None,
        input_ids_label: torch.Tensor = None,
        attention_mask: Optional[torch.Tensor] = None,
        attention_mask_label: Optional[torch.Tensor] = None,
    ):
        if self.is_pp_first_rank:
            batch_size, seq_len = input_ids.size()
            position_ids = torch.arange(
                seq_len, dtype=torch.long, device=input_ids.device
            )
            position_ids = position_ids.unsqueeze(0).expand(batch_size, seq_len)
            causal_mask = torch.tril(
                torch.ones(
                    (seq_len, seq_len), dtype=torch.bool, device=attention_mask.device
                )
            )
            causal_mask = causal_mask.unsqueeze(0).unsqueeze(
                0
            )  # shape (1, 1, seq_len, seq_len)
            pad_mask = attention_mask.unsqueeze(1).unsqueeze(2).bool()
            pad_mask = pad_mask.expand(-1, -1, seq_len, -1)
            attn_mask_4d = (
                causal_mask & pad_mask
            )  # shape: (batch_size, 1, seq_len, seq_len)
            attn_mask_4d = attn_mask_4d.expand(
                batch_size,
                self.config.num_attention_heads // self.tp_size,
                seq_len,
                seq_len,
            )
            output_tensor = self.model(
                input_ids=input_ids,
                attention_mask=attn_mask_4d.bool(),
                position_ids=position_ids,
                labels=input_ids_label if self.is_pp_last_rank else None,
            )
        else:
            output_tensor = self.model(
                decoder_input=None,
                input_ids=None,
                position_ids=None,
                attention_mask=None,
                labels=input_ids_label,
            )

        if self.is_pp_last_rank:
            losses = output_tensor.float()
            loss_mask = attention_mask_label.view(-1).float()
            loss = torch.sum(losses.view(-1) * loss_mask) / loss_mask.sum()
            return GenericOutputs(loss=loss)

        return output_tensor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import torch.nn as nn
from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
from transformers.models.roberta.modeling_roberta import (
    RobertaConfig,
    RobertaModel,
    RobertaLMHead,
)
from unitorch.models import GenericModel


class RobertaForClassification(GenericModel):
    def __init__(
        self,
        config_path: str,
        num_classes: Optional[int] = 1,
        gradient_checkpointing: Optional[bool] = False,
    ):
        """
        Initializes a RobertaForClassification model.

        Args:
            config_path (str): The path to the model configuration file.
            num_classes (int, optional): The number of classes for classification. Defaults to 1.
            gradient_checkpointing (bool, optional): Whether to use gradient checkpointing. Defaults to False.
        """
        super().__init__()
        self.config = RobertaConfig.from_json_file(config_path)
        self.config.gradient_checkpointing = gradient_checkpointing
        self.roberta = RobertaModel(self.config)
        self.dropout = nn.Dropout(self.config.hidden_dropout_prob)
        self.classifier = nn.Linear(self.config.hidden_size, num_classes)
        self.init_weights()

    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        token_type_ids: Optional[torch.Tensor] = None,
        position_ids: Optional[torch.Tensor] = None,
    ):
        """
        Performs forward pass of the RobertaForClassification model.

        Args:
            input_ids (torch.Tensor): Tensor of input token IDs.
            attention_mask (torch.Tensor, optional): Tensor of attention mask. Defaults to None.
            token_type_ids (torch.Tensor, optional): Tensor of token type IDs. Defaults to None.
            position_ids (torch.Tensor, optional): Tensor of position IDs. Defaults to None.

        Returns:
            (torch.Tensor):The model's logits.
        """
        outputs = self.roberta(
            input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            position_ids=position_ids,
        )
        pooled_output = outputs[1]

        pooled_output = self.dropout(pooled_output)
        logits = self.classifier(pooled_output)
        return logits


class RobertaForMaskLM(GenericModel):
    def __init__(
        self,
        config_path: str,
        gradient_checkpointing: Optional[bool] = False,
    ):
        """
        Initializes a RobertaForMaskLM model.

        Args:
            config_path (str): The path to the model configuration file.
            gradient_checkpointing (bool, optional): Whether to use gradient checkpointing. Defaults to False.
        """
        super().__init__()
        self.config = RobertaConfig.from_json_file(config_path)
        self.config.gradient_checkpointing = gradient_checkpointing
        self.roberta = RobertaModel(self.config, add_pooling_layer=False)
        self.lm_head = RobertaLMHead(self.config)
        self.lm_head.decoder.weight = self.roberta.embeddings.word_embeddings.weight
        self.init_weights()

    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        token_type_ids: Optional[torch.Tensor] = None,
        position_ids: Optional[torch.Tensor] = None,
    ):
        """
        Performs forward pass of the RobertaForMaskLM model.

        Args:
            input_ids (torch.Tensor): Tensor of input token IDs.
            attention_mask (torch.Tensor, optional): Tensor of attention mask. Defaults to None.
            token_type_ids (torch.Tensor, optional): Tensor of token type IDs. Defaults to None.
            position_ids (torch.Tensor, optional): Tensor of position IDs. Defaults to None.

        Returns:
            (torch.Tensor):The model's logits.
        """
        outputs = self.roberta(
            input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            position_ids=position_ids,
        )
        sequence_output = outputs[0]
        logits = self.lm_head(sequence_output)
        return logits



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import io
import requests
import time
import base64
import json
import logging
import torch
import torch.nn as nn
import numpy as np
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from random import random

from unitorch.cli import (
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)


class RandomProcessor:
    def __init__(
        self,
    ):
        pass

    @classmethod
    @add_default_section_for_init("core/process/random")
    def from_core_configure(cls, config, **kwargs):
        pass

    @register_process("core/process/random/float")
    def _float(
        self,
        low: float = 0.0,
        high: float = 1.0,
    ):
        return random() * (high - low) + low

    @register_process("core/process/random/int")
    def _int(
        self,
        low: int = 0,
        high: int = 100,
    ):
        return int(random() * (high - low) + low)

    @register_process("core/process/random/choice")
    def _choice(
        self,
        choices: List[Any],
        probs: Optional[List[float]] = None,
    ):
        if probs is None:
            probs = [1.0 / len(choices)] * len(choices)
        return np.random.choice(choices, p=probs)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torch import autocast

from unitorch.models.diffusers import (
    WanForText2VideoGeneration as _WanForText2VideoGeneration,
    WanForImage2VideoGeneration as _WanForImage2VideoGeneration,
)
from unitorch.utils import (
    pop_value,
    nested_dict_value,
    is_bfloat16_available,
    is_cuda_available,
)
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_model,
)
from unitorch.cli.models import DiffusionOutputs, LossOutputs
from unitorch.cli.models import diffusion_model_decorator
from unitorch.cli.models.diffusers import (
    pretrained_stable_infos,
    pretrained_stable_extensions_infos,
    load_weight,
)


@register_model("core/model/diffusers/text2video/wan", diffusion_model_decorator)
class WanForText2VideoGeneration(_WanForText2VideoGeneration):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        snr_gamma: Optional[float] = 5.0,
        seed: Optional[int] = 1123,
        gradient_checkpointing: Optional[bool] = True,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            snr_gamma=snr_gamma,
            seed=seed,
            gradient_checkpointing=gradient_checkpointing,
        )

    @classmethod
    @add_default_section_for_init("core/model/diffusers/text2video/wan")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/model/diffusers/text2video/wan")
        pretrained_name = config.getoption("pretrained_name", "wan-v2.1-t2v-1.3b")
        pretrained_infos = nested_dict_value(pretrained_stable_infos, pretrained_name)

        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_infos, "transformer", "config"),
        )
        config_path = cached_path(config_path)

        text_config_path = config.getoption("text_config_path", None)
        text_config_path = pop_value(
            text_config_path,
            nested_dict_value(pretrained_infos, "text", "config"),
        )
        text_config_path = cached_path(text_config_path)

        vae_config_path = config.getoption("vae_config_path", None)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrained_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)

        scheduler_config_path = config.getoption("scheduler_config_path", None)
        scheduler_config_path = pop_value(
            scheduler_config_path,
            nested_dict_value(pretrained_infos, "scheduler"),
        )
        scheduler_config_path = cached_path(scheduler_config_path)

        quant_config_path = config.getoption("quant_config_path", None)
        if quant_config_path is not None:
            quant_config_path = cached_path(quant_config_path)

        in_channels = config.getoption("in_channels", None)
        out_channels = config.getoption("out_channels", None)
        num_train_timesteps = config.getoption("num_train_timesteps", 1000)
        num_infer_timesteps = config.getoption("num_infer_timesteps", 50)
        freeze_vae_encoder = config.getoption("freeze_vae_encoder", True)
        freeze_text_encoder = config.getoption("freeze_text_encoder", True)
        snr_gamma = config.getoption("snr_gamma", 5.0)
        seed = config.getoption("seed", 1123)
        gradient_checkpointing = config.getoption("gradient_checkpointing", True)

        inst = cls(
            config_path=config_path,
            text_config_path=text_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            snr_gamma=snr_gamma,
            seed=seed,
            gradient_checkpointing=gradient_checkpointing,
        )

        weight_path = config.getoption("pretrained_weight_path", None)

        state_dict = None
        if weight_path is None and pretrained_infos is not None:
            state_dict = [
                load_weight(
                    nested_dict_value(pretrained_infos, "transformer", "weight"),
                    prefix_keys={"": "transformer."},
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "text", "weight"),
                    prefix_keys={"": "text."},
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "vae", "weight"),
                    prefix_keys={"": "vae."},
                ),
            ]

        inst.from_pretrained(weight_path, state_dict=state_dict)

        pretrained_lora_names = config.getoption("pretrained_lora_names", None)
        pretrained_lora_weights = config.getoption("pretrained_lora_weights", 1.0)
        pretrained_lora_alphas = config.getoption("pretrained_lora_alphas", 32.0)

        if isinstance(pretrained_lora_names, str):
            pretrained_lora_weights_path = nested_dict_value(
                pretrained_stable_extensions_infos,
                pretrained_lora_names,
                "lora",
                "weight",
            )
        elif isinstance(pretrained_lora_names, list):
            pretrained_lora_weights_path = [
                nested_dict_value(
                    pretrained_stable_extensions_infos, name, "lora", "weight"
                )
                for name in pretrained_lora_names
            ]
        else:
            pretrained_lora_weights_path = None

        lora_weights_path = config.getoption(
            "pretrained_lora_weights_path", pretrained_lora_weights_path
        )
        if lora_weights_path is not None:
            inst.load_lora_weights(
                lora_files=lora_weights_path,
                lora_weights=pretrained_lora_weights,
                lora_alphas=pretrained_lora_alphas,
                replace_keys={},
                save_base_state=False,
            )
        return inst

    @autocast(
        device_type=("cuda" if torch.cuda.is_available() else "cpu"),
        dtype=(torch.bfloat16 if is_bfloat16_available() else torch.float32),
    )
    def forward(
        self,
        input_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
    ):
        loss = super().forward(
            input_ids=input_ids,
            pixel_values=pixel_values,
            attention_mask=attention_mask,
        )
        return LossOutputs(loss=loss)

    @add_default_section_for_function("core/model/diffusers/text2video/wan")
    @autocast(
        device_type=("cuda" if torch.cuda.is_available() else "cpu"),
        dtype=(torch.bfloat16 if is_bfloat16_available() else torch.float32),
    )
    def generate(
        self,
        input_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        height: Optional[int] = 480,
        width: Optional[int] = 832,
        num_frames: Optional[int] = 81,
        guidance_scale: Optional[float] = 5.0,
    ):
        outputs = super().generate(
            input_ids=input_ids,
            negative_input_ids=negative_input_ids,
            attention_mask=attention_mask,
            negative_attention_mask=negative_attention_mask,
            height=height,
            width=width,
            num_frames=num_frames,
            guidance_scale=guidance_scale,
        )

        return DiffusionOutputs(outputs=outputs.frames.float())


@register_model("core/model/diffusers/image2video/wan", diffusion_model_decorator)
class WanForImage2VideoGeneration(_WanForImage2VideoGeneration):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        image_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        snr_gamma: Optional[float] = 5.0,
        seed: Optional[int] = 1123,
        gradient_checkpointing: Optional[bool] = True,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            image_config_path=image_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            snr_gamma=snr_gamma,
            seed=seed,
            gradient_checkpointing=gradient_checkpointing,
        )

    @classmethod
    @add_default_section_for_init("core/model/diffusers/image2video/wan")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/model/diffusers/image2video/wan")
        pretrained_name = config.getoption("pretrained_name", "wan-v2.1-i2v-14b-480p")
        pretrained_infos = nested_dict_value(pretrained_stable_infos, pretrained_name)

        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_infos, "transformer", "config"),
        )
        config_path = cached_path(config_path)

        text_config_path = config.getoption("text_config_path", None)
        text_config_path = pop_value(
            text_config_path,
            nested_dict_value(pretrained_infos, "text", "config"),
        )
        text_config_path = cached_path(text_config_path)

        image_config_path = config.getoption("image_config_path", None)
        image_config_path = pop_value(
            image_config_path,
            nested_dict_value(pretrained_infos, "image", "config"),
        )
        image_config_path = cached_path(image_config_path)

        vae_config_path = config.getoption("vae_config_path", None)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrained_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)

        scheduler_config_path = config.getoption("scheduler_config_path", None)
        scheduler_config_path = pop_value(
            scheduler_config_path,
            nested_dict_value(pretrained_infos, "scheduler"),
        )
        scheduler_config_path = cached_path(scheduler_config_path)

        quant_config_path = config.getoption("quant_config_path", None)
        if quant_config_path is not None:
            quant_config_path = cached_path(quant_config_path)

        in_channels = config.getoption("in_channels", None)
        out_channels = config.getoption("out_channels", None)
        num_train_timesteps = config.getoption("num_train_timesteps", 1000)
        num_infer_timesteps = config.getoption("num_infer_timesteps", 50)
        freeze_vae_encoder = config.getoption("freeze_vae_encoder", True)
        freeze_text_encoder = config.getoption("freeze_text_encoder", True)
        snr_gamma = config.getoption("snr_gamma", 5.0)
        seed = config.getoption("seed", 1123)
        gradient_checkpointing = config.getoption("gradient_checkpointing", True)

        inst = cls(
            config_path=config_path,
            text_config_path=text_config_path,
            image_config_path=image_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            snr_gamma=snr_gamma,
            seed=seed,
            gradient_checkpointing=gradient_checkpointing,
        )

        weight_path = config.getoption("pretrained_weight_path", None)

        state_dict = None
        if weight_path is None and pretrained_infos is not None:
            state_dict = [
                load_weight(
                    nested_dict_value(pretrained_infos, "transformer", "weight"),
                    prefix_keys={"": "transformer."},
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "text", "weight"),
                    prefix_keys={"": "text."},
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "image", "weight"),
                    prefix_keys={"": "image."},
                ),
                load_weight(
                    nested_dict_value(pretrained_infos, "vae", "weight"),
                    prefix_keys={"": "vae."},
                ),
            ]

        elif weight_path is not None:
            state_dict = load_weight(weight_path)

        if state_dict is not None:
            inst.from_pretrained(state_dict=state_dict)

        pretrained_lora_names = config.getoption("pretrained_lora_names", None)
        pretrained_lora_weights = config.getoption("pretrained_lora_weights", 1.0)
        pretrained_lora_alphas = config.getoption("pretrained_lora_alphas", 32.0)

        if isinstance(pretrained_lora_names, str):
            pretrained_lora_weights_path = nested_dict_value(
                pretrained_stable_extensions_infos,
                pretrained_lora_names,
                "lora",
                "weight",
            )
        elif isinstance(pretrained_lora_names, list):
            pretrained_lora_weights_path = [
                nested_dict_value(
                    pretrained_stable_extensions_infos, name, "lora", "weight"
                )
                for name in pretrained_lora_names
            ]
        else:
            pretrained_lora_weights_path = None

        lora_weights_path = config.getoption(
            "pretrained_lora_weights_path", pretrained_lora_weights_path
        )
        if lora_weights_path is not None:
            inst.load_lora_weights(
                lora_files=lora_weights_path,
                lora_weights=pretrained_lora_weights,
                lora_alphas=pretrained_lora_alphas,
                replace_keys={},
                save_base_state=False,
            )
        return inst

    @autocast(
        device_type=("cuda" if torch.cuda.is_available() else "cpu"),
        dtype=(torch.bfloat16 if is_bfloat16_available() else torch.float32),
    )
    def forward(
        self,
        pixel_values: torch.Tensor,
        condition_pixel_values: torch.Tensor,
        vae_pixel_values: torch.Tensor,
        input_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
    ):
        loss = super().forward(
            pixel_values=pixel_values,
            condition_pixel_values=condition_pixel_values,
            vae_pixel_values=vae_pixel_values,
            input_ids=input_ids,
            attention_mask=attention_mask,
        )
        return LossOutputs(loss=loss)

    @add_default_section_for_function("core/model/diffusers/image2video/wan")
    @autocast(
        device_type=("cuda" if torch.cuda.is_available() else "cpu"),
        dtype=(torch.bfloat16 if is_bfloat16_available() else torch.float32),
    )
    def generate(
        self,
        input_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        condition_pixel_values: torch.Tensor,
        vae_pixel_values: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        num_frames: Optional[int] = 81,
        guidance_scale: Optional[float] = 5.0,
    ):
        outputs = super().generate(
            input_ids=input_ids,
            negative_input_ids=negative_input_ids,
            condition_pixel_values=condition_pixel_values,
            vae_pixel_values=vae_pixel_values,
            attention_mask=attention_mask,
            negative_attention_mask=negative_attention_mask,
            num_frames=num_frames,
            guidance_scale=guidance_scale,
        )

        return DiffusionOutputs(outputs=outputs.frames.float())



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import json
import random
import logging
import torch
import torch.nn.functional as F
from torch import nn
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers.models.grounding_dino import GroundingDinoConfig
from transformers.models.grounding_dino.modeling_grounding_dino import (
    GroundingDinoModel,
    GroundingDinoContrastiveEmbedding,
    GroundingDinoMLPPredictionHead,
)
from transformers.loss.loss_deformable_detr import (
    DeformableDetrHungarianMatcher as GroundingDinoHungarianMatcher,
    DeformableDetrImageLoss as GroundingDinoLoss,
)
from unitorch.models import GenericModel, GenericOutputs
from unitorch.utils import image_list_to_tensor


class GroundingDinoForDetection(GenericModel):
    replace_keys_in_state_dict = {}

    def __init__(
        self,
        config_path: str,
    ):
        super().__init__()
        self.config = GroundingDinoConfig.from_json_file(config_path)
        self.model = GroundingDinoModel(self.config)
        _class_embed = GroundingDinoContrastiveEmbedding(self.config)

        if self.config.decoder_bbox_embed_share:
            _bbox_embed = GroundingDinoMLPPredictionHead(
                input_dim=self.config.d_model,
                hidden_dim=self.config.d_model,
                output_dim=4,
                num_layers=3,
            )
            self.bbox_embed = nn.ModuleList(
                [_bbox_embed for _ in range(self.config.decoder_layers)]
            )
        else:
            for _ in range(self.config.decoder_layers):
                _bbox_embed = GroundingDinoMLPPredictionHead(
                    input_dim=self.config.d_model,
                    hidden_dim=self.config.d_model,
                    output_dim=4,
                    num_layers=3,
                )
                self.bbox_embed = nn.ModuleList(
                    [_bbox_embed for _ in range(self.config.decoder_layers)]
                )
        self.class_embed = nn.ModuleList(
            [_class_embed for _ in range(self.config.decoder_layers)]
        )
        # hack for box-refinement
        self.model.decoder.bbox_embed = self.bbox_embed
        # hack implementation for two-stage
        self.model.decoder.class_embed = self.class_embed
        self.init_weights()

        self.enable_auxiliary_loss = self.config.auxiliary_loss
        self.matcher = GroundingDinoHungarianMatcher(
            class_cost=self.config.class_cost,
            bbox_cost=self.config.bbox_cost,
            giou_cost=self.config.giou_cost,
        )
        # Second: create the criterion
        self.losses = ["labels", "boxes", "cardinality"]
        self.criterion = GroundingDinoLoss(
            matcher=self.matcher,
            num_classes=self.config.num_labels,
            focal_alpha=self.config.focal_alpha,
            losses=self.losses,
        )

    @property
    def dtype(self):
        """
        `torch.dtype`: which dtype the parameters are (assuming that all the parameters are the same dtype).
        """

        return next(self.parameters()).dtype

    @property
    def device(self):
        """
        `torch.device`: The device on which the module is (assuming that all the module parameters are on the same device).
        """
        return next(self.parameters()).device

    def _set_aux_loss(self, outputs_class, outputs_coord):
        # this is a workaround to make torchscript happy, as torchscript
        # doesn't support dictionary with non-homogeneous values, such
        # as a dict having both a Tensor and a list.
        return [
            {"logits": a, "pred_boxes": b}
            for a, b in zip(outputs_class[:-1], outputs_coord[:-1])
        ]

    def forward(
        self,
        pixel_values: torch.Tensor,
        input_ids: torch.Tensor,
        token_type_ids: torch.Tensor,
        attention_mask: torch.Tensor,
        bboxes: Union[List[torch.Tensor], torch.Tensor],
        classes: Union[List[torch.Tensor], torch.Tensor],
    ):
        outputs = self.model(
            pixel_values=pixel_values,
            input_ids=input_ids,
            token_type_ids=token_type_ids,
            attention_mask=attention_mask,
            output_attentions=False,
            output_hidden_states=False,
            return_dict=True,
        )
        enc_text_hidden_state = outputs.encoder_last_hidden_state_text
        hidden_states = outputs.intermediate_hidden_states
        init_reference_points = outputs.init_reference_points
        inter_references_points = outputs.intermediate_reference_points
        outputs_classes = []
        outputs_coords = []
        num_levels = hidden_states.shape[1]
        for level in range(num_levels):
            if level == 0:
                reference = init_reference_points
            else:
                reference = inter_references_points[:, level - 1]
            reference = torch.special.logit(reference, eps=1e-5)
            outputs_class = self.class_embed[level](
                vision_hidden_state=hidden_states[:, level],
                text_hidden_state=enc_text_hidden_state,
                text_token_mask=attention_mask.bool(),
            )
            delta_bbox = self.bbox_embed[level](hidden_states[:, level])

            reference_coordinates = reference.shape[-1]
            if reference_coordinates == 4:
                outputs_coord_logits = delta_bbox + reference
            elif reference_coordinates == 2:
                delta_bbox[..., :2] += reference
                outputs_coord_logits = delta_bbox
            else:
                raise ValueError(
                    f"reference.shape[-1] should be 4 or 2, but got {reference.shape[-1]}"
                )
            outputs_coord = outputs_coord_logits.sigmoid()
            outputs_classes.append(outputs_class)
            outputs_coords.append(outputs_coord)
        outputs_class = torch.stack(outputs_classes)
        outputs_coord = torch.stack(outputs_coords)

        logits = outputs_class[-1]
        pred_boxes = outputs_coord[-1]

        loss, loss_dict, auxiliary_outputs = None, None, None
        # Third: compute the losses, based on outputs and labels
        outputs_loss = {}
        outputs_loss["logits"] = logits
        outputs_loss["pred_boxes"] = pred_boxes

        xyxy2xywh = lambda x: torch.stack(
            [
                (x[..., 0] + x[..., 2]) / 2,
                (x[..., 1] + x[..., 3]) / 2,
                x[..., 2] - x[..., 0],
                x[..., 3] - x[..., 1],
            ],
            -1,
        )
        bboxes = [xyxy2xywh(bbox) for bbox in bboxes]
        labels = [{"class_labels": c, "boxes": b} for b, c in zip(bboxes, classes)]

        if self.enable_auxiliary_loss:
            auxiliary_outputs = self._set_aux_loss(outputs_class, outputs_coord)
            outputs_loss["auxiliary_outputs"] = auxiliary_outputs
        if self.config.two_stage:
            enc_outputs_coord = outputs[-1].sigmoid()
            outputs_loss["enc_outputs"] = {
                "logits": outputs[-2],
                "pred_boxes": enc_outputs_coord,
            }

        loss_dict = self.criterion(outputs_loss, labels)
        # Fourth: compute total loss, as a weighted sum of the various losses
        weight_dict = {"loss_ce": 1, "loss_bbox": self.config.bbox_loss_coefficient}
        weight_dict["loss_giou"] = self.config.giou_loss_coefficient
        if self.config.auxiliary_loss:
            aux_weight_dict = {}
            for i in range(self.config.decoder_layers - 1):
                aux_weight_dict.update({k + f"_{i}": v for k, v in weight_dict.items()})
            weight_dict.update(aux_weight_dict)
        loss = sum(
            loss_dict[k] * weight_dict[k] for k in loss_dict.keys() if k in weight_dict
        )
        return loss

    @torch.no_grad()
    def detect(
        self,
        pixel_values: torch.Tensor,
        input_ids: torch.Tensor,
        token_type_ids: torch.Tensor,
        attention_mask: torch.Tensor,
        norm_bboxes: Optional[bool] = False,
        text_threshold: Optional[float] = 0.25,
        box_threshold: Optional[float] = 0.25,
    ):
        h, w = pixel_values.shape[-2:]
        outputs = self.model(
            pixel_values=pixel_values,
            input_ids=input_ids,
            token_type_ids=token_type_ids,
            attention_mask=attention_mask,
            output_attentions=False,
            output_hidden_states=False,
            return_dict=True,
        )
        enc_text_hidden_state = outputs.encoder_last_hidden_state_text
        hidden_states = outputs.intermediate_hidden_states
        init_reference_points = outputs.init_reference_points
        inter_references_points = outputs.intermediate_reference_points
        outputs_classes = []
        outputs_coords = []
        num_levels = hidden_states.shape[1]
        for level in range(num_levels):
            if level == 0:
                reference = init_reference_points
            else:
                reference = inter_references_points[:, level - 1]
            reference = torch.special.logit(reference, eps=1e-5)
            outputs_class = self.class_embed[level](
                vision_hidden_state=hidden_states[:, level],
                text_hidden_state=enc_text_hidden_state,
                text_token_mask=attention_mask.bool(),
            )
            delta_bbox = self.bbox_embed[level](hidden_states[:, level])

            reference_coordinates = reference.shape[-1]
            if reference_coordinates == 4:
                outputs_coord_logits = delta_bbox + reference
            elif reference_coordinates == 2:
                delta_bbox[..., :2] += reference
                outputs_coord_logits = delta_bbox
            else:
                raise ValueError(
                    f"reference.shape[-1] should be 4 or 2, but got {reference.shape[-1]}"
                )
            outputs_coord = outputs_coord_logits.sigmoid()
            outputs_classes.append(outputs_class)
            outputs_coords.append(outputs_coord)
        outputs_class = torch.stack(outputs_classes)
        outputs_coord = torch.stack(outputs_coords)

        logits = outputs_class[-1]
        pred_boxes = outputs_coord[-1]

        probs = torch.sigmoid(logits)
        scores = torch.max(probs, dim=-1)[0]
        xywh2xyxy = lambda x: torch.stack(
            [
                x[..., 0] - x[..., 2] / 2,
                x[..., 1] - x[..., 3] / 2,
                x[..., 0] + x[..., 2] / 2,
                x[..., 1] + x[..., 3] / 2,
            ],
            -1,
        )
        pred_boxes = [xywh2xyxy(bbox) for bbox in pred_boxes]
        if not norm_bboxes:
            bboxes = [b * torch.tensor([w, h, w, h]).to(b) for b in pred_boxes]
        else:
            bboxes = pred_boxes

        class_ids = input_ids.unsqueeze(1).expand(-1, bboxes[0].shape[0], -1)
        max_len = min(input_ids.shape[-1], logits.shape[-1])
        bboxes, scores, classes = list(
            zip(
                *[
                    (
                        b[s > box_threshold],
                        s[s > box_threshold],
                        (
                            (p[s > box_threshold] > text_threshold).float()[:, :max_len]
                            * c[s > box_threshold].float()[:, :max_len]
                        ).long(),
                    )
                    for b, s, p, c in zip(bboxes, scores, probs, class_ids)
                ]
            )
        )

        outputs = dict(
            {
                "bboxes": list(bboxes),
                "scores": list(scores),
                "classes": list(classes),
            }
        )
        return GenericOutputs(outputs)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import math
import numpy as np
import matplotlib
import cv2


eps = 0.01


def smart_resize(x, s):
    Ht, Wt = s
    if x.ndim == 2:
        Ho, Wo = x.shape
        Co = 1
    else:
        Ho, Wo, Co = x.shape
    if Co == 3 or Co == 1:
        k = float(Ht + Wt) / float(Ho + Wo)
        return cv2.resize(
            x,
            (int(Wt), int(Ht)),
            interpolation=cv2.INTER_AREA if k < 1 else cv2.INTER_LANCZOS4,
        )
    else:
        return np.stack([smart_resize(x[:, :, i], s) for i in range(Co)], axis=2)


def smart_resize_k(x, fx, fy):
    if x.ndim == 2:
        Ho, Wo = x.shape
        Co = 1
    else:
        Ho, Wo, Co = x.shape
    Ht, Wt = Ho * fy, Wo * fx
    if Co == 3 or Co == 1:
        k = float(Ht + Wt) / float(Ho + Wo)
        return cv2.resize(
            x,
            (int(Wt), int(Ht)),
            interpolation=cv2.INTER_AREA if k < 1 else cv2.INTER_LANCZOS4,
        )
    else:
        return np.stack([smart_resize_k(x[:, :, i], fx, fy) for i in range(Co)], axis=2)


def padRightDownCorner(img, stride, padValue):
    h = img.shape[0]
    w = img.shape[1]

    pad = 4 * [None]
    pad[0] = 0  # up
    pad[1] = 0  # left
    pad[2] = 0 if (h % stride == 0) else stride - (h % stride)  # down
    pad[3] = 0 if (w % stride == 0) else stride - (w % stride)  # right

    img_padded = img
    pad_up = np.tile(img_padded[0:1, :, :] * 0 + padValue, (pad[0], 1, 1))
    img_padded = np.concatenate((pad_up, img_padded), axis=0)
    pad_left = np.tile(img_padded[:, 0:1, :] * 0 + padValue, (1, pad[1], 1))
    img_padded = np.concatenate((pad_left, img_padded), axis=1)
    pad_down = np.tile(img_padded[-2:-1, :, :] * 0 + padValue, (pad[2], 1, 1))
    img_padded = np.concatenate((img_padded, pad_down), axis=0)
    pad_right = np.tile(img_padded[:, -2:-1, :] * 0 + padValue, (1, pad[3], 1))
    img_padded = np.concatenate((img_padded, pad_right), axis=1)

    return img_padded, pad


def transfer(model, model_weights):
    transfered_model_weights = {}
    for weights_name in model.state_dict().keys():
        transfered_model_weights[weights_name] = model_weights[
            ".".join(weights_name.split(".")[1:])
        ]
    return transfered_model_weights


def draw_bodypose(canvas, candidate, subset):
    H, W, C = canvas.shape
    candidate = np.array(candidate)
    subset = np.array(subset)

    stickwidth = 4

    limbSeq = [
        [2, 3],
        [2, 6],
        [3, 4],
        [4, 5],
        [6, 7],
        [7, 8],
        [2, 9],
        [9, 10],
        [10, 11],
        [2, 12],
        [12, 13],
        [13, 14],
        [2, 1],
        [1, 15],
        [15, 17],
        [1, 16],
        [16, 18],
        [3, 17],
        [6, 18],
    ]

    colors = [
        [255, 0, 0],
        [255, 85, 0],
        [255, 170, 0],
        [255, 255, 0],
        [170, 255, 0],
        [85, 255, 0],
        [0, 255, 0],
        [0, 255, 85],
        [0, 255, 170],
        [0, 255, 255],
        [0, 170, 255],
        [0, 85, 255],
        [0, 0, 255],
        [85, 0, 255],
        [170, 0, 255],
        [255, 0, 255],
        [255, 0, 170],
        [255, 0, 85],
    ]

    for i in range(17):
        for n in range(len(subset)):
            index = subset[n][np.array(limbSeq[i]) - 1]
            if -1 in index:
                continue
            Y = candidate[index.astype(int), 0] * float(W)
            X = candidate[index.astype(int), 1] * float(H)
            mX = np.mean(X)
            mY = np.mean(Y)
            length = ((X[0] - X[1]) ** 2 + (Y[0] - Y[1]) ** 2) ** 0.5
            angle = math.degrees(math.atan2(X[0] - X[1], Y[0] - Y[1]))
            polygon = cv2.ellipse2Poly(
                (int(mY), int(mX)), (int(length / 2), stickwidth), int(angle), 0, 360, 1
            )
            cv2.fillConvexPoly(canvas, polygon, colors[i])

    canvas = (canvas * 0.6).astype(np.uint8)

    for i in range(18):
        for n in range(len(subset)):
            index = int(subset[n][i])
            if index == -1:
                continue
            x, y = candidate[index][0:2]
            x = int(x * W)
            y = int(y * H)
            cv2.circle(canvas, (int(x), int(y)), 4, colors[i], thickness=-1)

    return canvas


def draw_handpose(canvas, all_hand_peaks):
    H, W, C = canvas.shape

    edges = [
        [0, 1],
        [1, 2],
        [2, 3],
        [3, 4],
        [0, 5],
        [5, 6],
        [6, 7],
        [7, 8],
        [0, 9],
        [9, 10],
        [10, 11],
        [11, 12],
        [0, 13],
        [13, 14],
        [14, 15],
        [15, 16],
        [0, 17],
        [17, 18],
        [18, 19],
        [19, 20],
    ]

    for peaks in all_hand_peaks:
        peaks = np.array(peaks)

        for ie, e in enumerate(edges):
            x1, y1 = peaks[e[0]]
            x2, y2 = peaks[e[1]]
            x1 = int(x1 * W)
            y1 = int(y1 * H)
            x2 = int(x2 * W)
            y2 = int(y2 * H)
            if x1 > eps and y1 > eps and x2 > eps and y2 > eps:
                cv2.line(
                    canvas,
                    (x1, y1),
                    (x2, y2),
                    matplotlib.colors.hsv_to_rgb([ie / float(len(edges)), 1.0, 1.0])
                    * 255,
                    thickness=2,
                )

        for i, keyponit in enumerate(peaks):
            x, y = keyponit
            x = int(x * W)
            y = int(y * H)
            if x > eps and y > eps:
                cv2.circle(canvas, (x, y), 4, (0, 0, 255), thickness=-1)
    return canvas


def draw_facepose(canvas, all_lmks):
    H, W, C = canvas.shape
    for lmks in all_lmks:
        lmks = np.array(lmks)
        for lmk in lmks:
            x, y = lmk
            x = int(x * W)
            y = int(y * H)
            if x > eps and y > eps:
                cv2.circle(canvas, (x, y), 3, (255, 255, 255), thickness=-1)
    return canvas


# detect hand according to body pose keypoints
# please refer to https://github.com/CMU-Perceptual-Computing-Lab/openpose/blob/master/src/openpose/hand/handDetector.cpp
def handDetect(candidate, subset, oriImg):
    # right hand: wrist 4, elbow 3, shoulder 2
    # left hand: wrist 7, elbow 6, shoulder 5
    ratioWristElbow = 0.33
    detect_result = []
    image_height, image_width = oriImg.shape[0:2]
    for person in subset.astype(int):
        # if any of three not detected
        has_left = np.sum(person[[5, 6, 7]] == -1) == 0
        has_right = np.sum(person[[2, 3, 4]] == -1) == 0
        if not (has_left or has_right):
            continue
        hands = []
        # left hand
        if has_left:
            left_shoulder_index, left_elbow_index, left_wrist_index = person[[5, 6, 7]]
            x1, y1 = candidate[left_shoulder_index][:2]
            x2, y2 = candidate[left_elbow_index][:2]
            x3, y3 = candidate[left_wrist_index][:2]
            hands.append([x1, y1, x2, y2, x3, y3, True])
        # right hand
        if has_right:
            right_shoulder_index, right_elbow_index, right_wrist_index = person[
                [2, 3, 4]
            ]
            x1, y1 = candidate[right_shoulder_index][:2]
            x2, y2 = candidate[right_elbow_index][:2]
            x3, y3 = candidate[right_wrist_index][:2]
            hands.append([x1, y1, x2, y2, x3, y3, False])

        for x1, y1, x2, y2, x3, y3, is_left in hands:
            # pos_hand = pos_wrist + ratio * (pos_wrist - pos_elbox) = (1 + ratio) * pos_wrist - ratio * pos_elbox
            # handRectangle.x = posePtr[wrist*3] + ratioWristElbow * (posePtr[wrist*3] - posePtr[elbow*3]);
            # handRectangle.y = posePtr[wrist*3+1] + ratioWristElbow * (posePtr[wrist*3+1] - posePtr[elbow*3+1]);
            # const auto distanceWristElbow = getDistance(poseKeypoints, person, wrist, elbow);
            # const auto distanceElbowShoulder = getDistance(poseKeypoints, person, elbow, shoulder);
            # handRectangle.width = 1.5f * fastMax(distanceWristElbow, 0.9f * distanceElbowShoulder);
            x = x3 + ratioWristElbow * (x3 - x2)
            y = y3 + ratioWristElbow * (y3 - y2)
            distanceWristElbow = math.sqrt((x3 - x2) ** 2 + (y3 - y2) ** 2)
            distanceElbowShoulder = math.sqrt((x2 - x1) ** 2 + (y2 - y1) ** 2)
            width = 1.5 * max(distanceWristElbow, 0.9 * distanceElbowShoulder)
            # x-y refers to the center --> offset to topLeft point
            # handRectangle.x -= handRectangle.width / 2.f;
            # handRectangle.y -= handRectangle.height / 2.f;
            x -= width / 2
            y -= width / 2  # width = height
            # overflow the image
            if x < 0:
                x = 0
            if y < 0:
                y = 0
            width1 = width
            width2 = width
            if x + width > image_width:
                width1 = image_width - x
            if y + width > image_height:
                width2 = image_height - y
            width = min(width1, width2)
            # the max hand box value is 20 pixels
            if width >= 20:
                detect_result.append([int(x), int(y), int(width), is_left])

    """
    return value: [[x, y, w, True if left hand else False]].
    width=height since the network require squared input.
    x, y is the coordinate of top left 
    """
    return detect_result


# Written by Lvmin
def faceDetect(candidate, subset, oriImg):
    # left right eye ear 14 15 16 17
    detect_result = []
    image_height, image_width = oriImg.shape[0:2]
    for person in subset.astype(int):
        has_head = person[0] > -1
        if not has_head:
            continue

        has_left_eye = person[14] > -1
        has_right_eye = person[15] > -1
        has_left_ear = person[16] > -1
        has_right_ear = person[17] > -1

        if not (has_left_eye or has_right_eye or has_left_ear or has_right_ear):
            continue

        head, left_eye, right_eye, left_ear, right_ear = person[[0, 14, 15, 16, 17]]

        width = 0.0
        x0, y0 = candidate[head][:2]

        if has_left_eye:
            x1, y1 = candidate[left_eye][:2]
            d = max(abs(x0 - x1), abs(y0 - y1))
            width = max(width, d * 3.0)

        if has_right_eye:
            x1, y1 = candidate[right_eye][:2]
            d = max(abs(x0 - x1), abs(y0 - y1))
            width = max(width, d * 3.0)

        if has_left_ear:
            x1, y1 = candidate[left_ear][:2]
            d = max(abs(x0 - x1), abs(y0 - y1))
            width = max(width, d * 1.5)

        if has_right_ear:
            x1, y1 = candidate[right_ear][:2]
            d = max(abs(x0 - x1), abs(y0 - y1))
            width = max(width, d * 1.5)

        x, y = x0, y0

        x -= width
        y -= width

        if x < 0:
            x = 0

        if y < 0:
            y = 0

        width1 = width * 2
        width2 = width * 2

        if x + width > image_width:
            width1 = image_width - x

        if y + width > image_height:
            width2 = image_height - y

        width = min(width1, width2)

        if width >= 20:
            detect_result.append([int(x), int(y), int(width)])

    return detect_result


# get max index of 2d array
def npmax(array):
    arrayindex = array.argmax(1)
    arrayvalue = array.max(1)
    i = arrayvalue.argmax()
    j = arrayindex[i]
    return i, j



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from PIL import Image
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.grounding_dino import (
    GroundingDinoProcessor as _GroundingDinoProcessor,
)
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli.models import (
    TensorsInputs,
    ListTensorsInputs,
    DetectionOutputs,
    DetectionTargets,
    SegmentationOutputs,
    SegmentationTargets,
)
from unitorch.cli import WriterOutputs
from unitorch.cli.models.grounding_dino import pretrained_grounding_dino_infos


class GroundingDinoProcessor(_GroundingDinoProcessor):
    def __init__(
        self,
        vocab_path: str,
        vision_config_path: str,
    ):
        super().__init__(
            vocab_path=vocab_path,
            vision_config_path=vision_config_path,
        )

    @classmethod
    @add_default_section_for_init("core/process/grounding_dino")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/process/grounding_dino")
        pretrained_name = config.getoption("pretrained_name", "grounding-dino-tiny")

        vocab_path = config.getoption("vocab_path", None)
        vocab_path = pop_value(
            vocab_path,
            nested_dict_value(
                pretrained_grounding_dino_infos, pretrained_name, "vocab"
            ),
        )
        vocab_path = cached_path(vocab_path)

        vision_config_path = config.getoption("vision_config_path", None)
        vision_config_path = pop_value(
            vision_config_path,
            nested_dict_value(
                pretrained_grounding_dino_infos, pretrained_name, "vision_config"
            ),
        )
        vision_config_path = cached_path(vision_config_path)

        return {
            "vocab_path": vocab_path,
            "vision_config_path": vision_config_path,
        }

    @register_process("core/process/grounding_dino/detection/inputs")
    def _detection_inputs(
        self,
        text: str,
        image: Union[Image.Image, str],
    ):
        outputs = super().detection_inputs(
            text=text,
            image=image,
        )

        return TensorsInputs(
            pixel_values=outputs.pixel_values,
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            token_type_ids=outputs.token_type_ids,
        )

    @register_process("core/process/grounding_dino/detection")
    def _detection(
        self,
        text: str,
        image: Union[Image.Image, str],
        bboxes: List[List[float]],
        classes: List[str],
        do_eval: Optional[bool] = False,
    ):
        outputs = super().detection(
            text=text,
            image=image,
            bboxes=bboxes,
            classes=classes,
        )
        if do_eval:
            new_h, new_w = outputs.pixel_values.size()[1:]
            bboxes = outputs.bboxes
            bboxes[:, 0] = bboxes[:, 0] * new_w
            bboxes[:, 1] = bboxes[:, 1] * new_h
            bboxes[:, 2] = bboxes[:, 2] * new_w
            bboxes[:, 3] = bboxes[:, 3] * new_h
            return TensorsInputs(
                pixel_values=outputs.pixel_values,
                input_ids=outputs.input_ids,
                attention_mask=outputs.attention_mask,
                token_type_ids=outputs.token_type_ids,
            ), DetectionTargets(
                bboxes=bboxes,
                classes=outputs.classes,
            )

        return TensorsInputs(
            pixel_values=outputs.pixel_values,
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            token_type_ids=outputs.token_type_ids,
        ), ListTensorsInputs(
            bboxes=outputs.bboxes,
            classes=outputs.classes,
        )

    @register_process("core/postprocess/grounding_dino/detection")
    def _post_dectection(self, outputs: DetectionOutputs):
        results = outputs.to_pandas()
        results["bboxes"] = [bboxes.tolist() for bboxes in outputs.bboxes]
        results["scores"] = [scores.tolist() for scores in outputs.scores]
        results["classes"] = [classes.tolist() for classes in outputs.classes]
        return WriterOutputs(results)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli import hf_endpoint_url

# pretrained infos
pretrained_kolors_infos = {
    "kolors-mps-overall": {
        "config": hf_endpoint_url(
            "/laion/CLIP-ViT-H-14-laion2B-s32B-b79K/resolve/main/config.json"
        ),
        "vision_config": hf_endpoint_url(
            "/laion/CLIP-ViT-H-14-laion2B-s32B-b79K/resolve/main/preprocessor_config.json"
        ),
        "vocab": hf_endpoint_url(
            "/laion/CLIP-ViT-H-14-laion2B-s32B-b79K/resolve/main/vocab.json"
        ),
        "merge": hf_endpoint_url(
            "/laion/CLIP-ViT-H-14-laion2B-s32B-b79K/resolve/main/merges.txt"
        ),
        "weight": hf_endpoint_url(
            "/datasets/fuliucansheng/hubfiles/resolve/main/kolors/pytorch_model.mps.overall.bin"
        ),
    },
}

pretrained_kolors_extensions_infos = {}

import unitorch.cli.models.kolors.modeling
import unitorch.cli.models.kolors.processing
from unitorch.cli.models.kolors.modeling import (
    KolorsMPSModel,
)
from unitorch.cli.models.kolors.processing import KolorsMPSProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import re
import torch
import pandas as pd
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import is_remote_url
from unitorch.models.llama import (
    LlamaForGeneration as _LlamaForGeneration,
)
from unitorch.models.llama import LlamaProcessor
from unitorch.utils import pop_value, nested_dict_value
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
)
from unitorch.cli import CoreConfigureParser, GenericScript
from unitorch.cli import register_script
from unitorch.cli.models.llama import (
    pretrained_llama_infos,
    pretrained_llama_extensions_infos,
)


class LlamaForGenerationPipeline(_LlamaForGeneration):
    def __init__(
        self,
        config_path: str,
        vocab_path: str,
        quant_config_path: Optional[str] = None,
        max_seq_length: Optional[int] = 512,
        max_gen_seq_length: Optional[int] = 512,
        weight_path: Optional[Union[str, List[str]]] = None,
        state_dict: Optional[Dict[str, Any]] = None,
        enable_cpu_offload: Optional[bool] = True,
        device: Optional[Union[str, int]] = "cpu",
    ):
        if device == "cpu":
            quant_config_path = None
        super().__init__(
            config_path=config_path,
            quant_config_path=quant_config_path,
        )
        self.processor = LlamaProcessor(
            vocab_path=vocab_path,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )
        self._device = "cpu" if device == "cpu" else int(device)

        self.from_pretrained(weight_path, state_dict=state_dict)
        self._enable_cpu_offload = enable_cpu_offload
        if not self._enable_cpu_offload and self._device != "cpu":
            self.to(device=self._device)
        self.eval()

    @classmethod
    @add_default_section_for_init("core/pipeline/llama")
    def from_core_configure(
        cls,
        config,
        pretrained_name: Optional[str] = None,
        config_path: Optional[str] = None,
        vocab_path: Optional[str] = None,
        quant_config_path: Optional[str] = None,
        pretrained_weight_path: Optional[str] = None,
        device: Optional[str] = None,
        **kwargs,
    ):
        config.set_default_section("core/pipeline/llama")
        pretrained_name = pretrained_name or config.getoption(
            "pretrained_name", "llama-7b"
        )

        config_path = config_path or config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_llama_infos, pretrained_name, "config"),
        )
        config_path = cached_path(config_path)

        vocab_path = vocab_path or config.getoption("vocab_path", None)
        vocab_path = pop_value(
            vocab_path,
            nested_dict_value(pretrained_llama_infos, pretrained_name, "vocab"),
        )
        vocab_path = cached_path(vocab_path)

        quant_config_path = quant_config_path or config.getoption(
            "quant_config_path", None
        )
        if quant_config_path is not None:
            quant_config_path = cached_path(quant_config_path)

        max_seq_length = config.getoption("max_seq_length", 512)
        max_gen_seq_length = config.getoption("max_gen_seq_length", 512)
        enable_cpu_offload = config.getoption("enable_cpu_offload", True)
        device = config.getoption("device", "cpu") if device is None else device
        pretrained_weight_path = pretrained_weight_path or config.getoption(
            "pretrained_weight_path", None
        )
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_llama_infos, pretrained_name, "weight"),
            check_none=False,
        )

        inst = cls(
            config_path,
            vocab_path=vocab_path,
            quant_config_path=quant_config_path,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
            weight_path=weight_path,
            enable_cpu_offload=enable_cpu_offload,
            device=device,
        )

        return inst

    @torch.no_grad()
    @add_default_section_for_function("core/pipeline/llama")
    def __call__(
        self,
        prompt: str,
        max_seq_length: Optional[int] = 512,
        num_beams: Optional[int] = 2,
        decoder_start_token_id: Optional[int] = 1,
        decoder_end_token_id: Optional[Union[int, List[int]]] = [2],
        num_return_sequences: Optional[int] = 1,
        min_gen_seq_length: Optional[int] = 0,
        max_gen_seq_length: Optional[int] = 512,
        repetition_penalty: Optional[float] = 1.0,
        no_repeat_ngram_size: Optional[int] = 0,
        early_stopping: Optional[bool] = True,
        length_penalty: Optional[float] = 1.0,
        num_beam_groups: Optional[int] = 1,
        diversity_penalty: Optional[float] = 0.0,
        do_sample: Optional[bool] = False,
        temperature: Optional[float] = 1.0,
        top_k: Optional[int] = 50,
        top_p: Optional[float] = 1.0,
        lora_checkpoints: Optional[Union[str, List[str]]] = [],
        lora_weights: Optional[Union[float, List[float]]] = 1.0,
        lora_alphas: Optional[Union[float, List[float]]] = 32,
        lora_urls: Optional[Union[str, List[str]]] = [],
        lora_files: Optional[Union[str, List[str]]] = [],
    ):
        if self._enable_cpu_offload:
            self.to(self._device)
        inputs = self.processor.generation_inputs(
            text=prompt,
            max_seq_length=max_seq_length,
        )
        inputs = {k: v.unsqueeze(0) if v is not None else v for k, v in inputs.items()}
        inputs = {
            k: v.to(device=self._device) if v is not None else v
            for k, v in inputs.items()
        }
        if isinstance(lora_checkpoints, str):
            lora_checkpoints = [lora_checkpoints]
        if isinstance(lora_weights, float):
            lora_weights = [lora_weights]
        if isinstance(lora_alphas, float):
            lora_alphas = [lora_alphas]
        if isinstance(lora_urls, str):
            lora_urls = [lora_urls]
        if isinstance(lora_files, str):
            lora_files = [lora_files]

        assert (
            len(lora_checkpoints) == len(lora_weights)
            and len(lora_checkpoints) == len(lora_alphas)
            and len(lora_checkpoints) == len(lora_urls)
            and len(lora_checkpoints) == len(lora_files)
        )
        processed_lora_files, processed_lora_weights, processed_lora_alphas = [], [], []
        for ckpt, url, file, weight, alpha in zip(
            lora_checkpoints, lora_urls, lora_files, lora_weights, lora_alphas
        ):
            if ckpt is not None:
                lora_file = nested_dict_value(
                    pretrained_llama_extensions_infos, ckpt, "weight"
                )
                processed_lora_files.append(lora_file)
                processed_lora_weights.append(weight)
                processed_lora_alphas.append(alpha)
            elif url is not None and is_remote_url(url):
                processed_lora_files.append(url)
                processed_lora_weights.append(weight)
                processed_lora_alphas.append(alpha)
            elif file is not None:
                processed_lora_files.append(file)
                processed_lora_weights.append(weight)
                processed_lora_alphas.append(alpha)

        if len(processed_lora_files) > 0:
            self.load_lora_weights(
                processed_lora_files,
                lora_weights=processed_lora_weights,
                lora_alphas=processed_lora_alphas,
            )

        outputs = super().generate(
            input_ids=inputs["input_ids"],
            num_beams=num_beams,
            decoder_start_token_id=decoder_start_token_id,
            decoder_end_token_id=decoder_end_token_id,
            num_return_sequences=num_return_sequences,
            min_gen_seq_length=min_gen_seq_length,
            max_gen_seq_length=max_gen_seq_length,
            repetition_penalty=repetition_penalty,
            no_repeat_ngram_size=no_repeat_ngram_size,
            early_stopping=early_stopping,
            length_penalty=length_penalty,
            num_beam_groups=num_beam_groups,
            diversity_penalty=diversity_penalty,
            do_sample=do_sample,
            temperature=temperature,
            top_k=top_k,
            top_p=top_p,
        )
        self.unload_lora_weights()
        decoded = self.processor.detokenize(outputs.sequences)
        if self._enable_cpu_offload:
            self.to("cpu")
            torch.cuda.empty_cache()
        return decoded[0].strip()
